#!/bin/sh
# $Id: t-cltbind-0.sh,v 1.22 2011/05/03 18:42:11 ca Exp $
# Copyright (c) 2011 Sendmail, Inc. and its suppliers.
#	All rights reserved.
#
# By using this file, you agree to the terms and conditions set
# forth in the LICENSE file which can be found at the top level of
# the sendmail distribution.
#
# ----------------------------------------
# test client IP address binding, selection based on cltaddr and MAIL
# ----------------------------------------
#

SD=`dirname $0`
THISDIR=`pwd`
if test "${SD}" = "."
then
   SD=../chkmts
fi
. ${SD}/common.sh

chkcto smtpc COMMERCIAL
chkcto smtpc SC_SESS_FEATS

if test X"`uname`" != "XOSF1"
then
while getopts S:s:V FLAG
do
  case "${FLAG}" in
    S) SKIP="${OPTARG}";;
    s) SKIP="${SKIP} ${OPTARG}";;
    V) VERBOSE=true;;
  esac
done
shift `expr ${OPTIND} - 1`
fi

getip2 2

cd ../qmgr || exit 1

${CAT} > ${SMXCNF} <<EOF
smtpc { Log_Level = 13; remote_port=${SNKPORT}; wait_for_server=4;
}
smtps {
	Log_Level = 12;
	daemon_address = localhost:${SRVPORT};
	wait_for_server=4;
}
qmgr { Log_Level = 12; wait_for_server=4; wait_for_client=4;
  smtpc {
	source bind0 { ips = { $H2 } }
	source bind1 { ips = { 127.0.0.1 } }
  }
}
smar { Log_Level = 12; nameserver = ${NS}; }
EOF

# create mailertable (save old version?)
${CAT} > ${MT} <<EOF
local.dom	lmtp:
some.dom	[127.0.0.1]
other.tld	[127.0.0.1]
EOF

# ----------------------------------------
# test: send mails, check whether connect from smtpc to sink comes from right
# address based on client IP address
TEST=1
if echo " ${SKIP} X" | ${GREP} -v " ${TEST} " > /dev/null
then
ERR=false

${CAT} > ${QMGRCONF} <<EOF
smtpc_conf_cltaddr:127.0.0.1	source=bind0;
EOF
rm -f ${QMGRCONFMAP}
${MM} -t'	' -F ${QMGRCONFMAP} < ${QMGRCONF}

if ${SHELL} ${SD}/smtpcnf.sh -F ${SRVPORT} -Ssender@some.dom -Ruser@some.dom -E 1 -X 1 -A 1 -Z "smtpc2" -d 5 -Y "-d 8"
then
  if grep "connection from $H2" ${SRVL} >/dev/null
  then
    :
  else
    ERR=true
    $VERBOSE && echo "$0: missing connection from $H2"
  fi
else
  ERR=true
fi
if ${ERR}
then
   echo "$0: test ${TEST} failed"
   ERRS=`expr ${ERRS} + 1 `
   test X"${MTA_STOPONERROR}" != X && exit 1
fi
else
  ${VERBOSE} && echo "$0: test ${TEST} SKIPPED"
fi

# ----------------------------------------
# test: send mails, check whether connect from smtpc to sink comes from right
# address based on sender address
TEST=2
if echo " ${SKIP} X" | ${GREP} -v " ${TEST} " > /dev/null
then
ERR=false

${CAT} > ${QMGRCONF} <<EOF
smtpc_conf_mail:sender@some.dom	source=bind0;
EOF
rm -f ${QMGRCONFMAP}
${MM} -t'	' -F ${QMGRCONFMAP} < ${QMGRCONF}

if ${SHELL} ${SD}/smtpcnf.sh -F ${SRVPORT} -Ssender@some.dom -Ruser@some.dom -E 1 -X 1 -A 1 -Z "smtpc2" -d 5 -Y "-d 8"
then
  if grep "connection from $H2" ${SRVL} >/dev/null
  then
    :
  else
    ERR=true
    $VERBOSE && echo "$0: missing connection from $H2"
  fi
else
  ERR=true
fi
if ${ERR}
then
   echo "$0: test ${TEST} failed"
   ERRS=`expr ${ERRS} + 1 `
   test X"${MTA_STOPONERROR}" != X && exit 1
fi
else
  ${VERBOSE} && echo "$0: test ${TEST} SKIPPED"
fi

# ----------------------------------------
# test: send mails, check whether connect from smtpc to sink comes from right
# address based on sender domain
TEST=3
if echo " ${SKIP} X" | ${GREP} -v " ${TEST} " > /dev/null
then
ERR=false

${CAT} > ${QMGRCONF} <<EOF
smtpc_conf_mail:@.tld	source=bind0;
EOF
rm -f ${QMGRCONFMAP}
${MM} -t'	' -F ${QMGRCONFMAP} < ${QMGRCONF}

if ${SHELL} ${SD}/smtpcnf.sh -F ${SRVPORT} -Ssender@other.tld -Ruser@some.dom -E 1 -X 1 -A 1 -Z "smtpc2" -d 5 -Y "-d 8"
then
  if grep "connection from $H2" ${SRVL} >/dev/null
  then
    :
  else
    ERR=true
    $VERBOSE && echo "$0: missing connection from $H2"
  fi
else
  ERR=true
fi
if ${ERR}
then
   echo "$0: test ${TEST} failed"
   ERRS=`expr ${ERRS} + 1 `
   test X"${MTA_STOPONERROR}" != X && exit 1
fi
else
  ${VERBOSE} && echo "$0: test ${TEST} SKIPPED"
fi

# ----------------------------------------
# test: send mails, check whether connect from smtpc to sink comes from right
# address based on sender domain
TEST=4
if echo " ${SKIP} X" | ${GREP} -v " ${TEST} " > /dev/null
then
ERR=false

${CAT} > ${QMGRCONF} <<EOF
smtpc_conf_mail:@other.tld	source=bind1;
smtpc_conf_mail:sender@other.tld	source=bind0;
EOF
rm -f ${QMGRCONFMAP}
${MM} -t'	' -F ${QMGRCONFMAP} < ${QMGRCONF}

if ${SHELL} ${SD}/smtpcnf.sh -F ${SRVPORT} -E 2 -X 2 -A 2 -Z "smtpc2" -c "-fsender@other.tld -Ruser@some.dom -fs1@other.tld -m 2 -s 2" -d 5 -Y "-d 8"
then
  if grep "connection from $H2" ${SRVL} >/dev/null
  then
    CNT=`grep -c "connection from $H2" ${SRVL}`
    if test x"$CNT" != "x"
    then
      if test $CNT -ne 1
      then
        ERR=true
        $VERBOSE && echo "$0: too many connections from $H2"
      fi
    else
      ERR=true
      $VERBOSE && echo "$0: cannot count connections from $H2"
    fi
  else
    ERR=true
    $VERBOSE && echo "$0: missing connection from $H2"
  fi
else
  ERR=true
fi
if ${ERR}
then
   echo "$0: test ${TEST} failed"
   ERRS=`expr ${ERRS} + 1 `
   test X"${MTA_STOPONERROR}" != X && exit 1
fi
else
  ${VERBOSE} && echo "$0: test ${TEST} SKIPPED"
fi

# ----------------------------------------
# test: send many mails
TEST=5
if echo " ${SKIP} X" | ${GREP} -v " ${TEST} " > /dev/null
then
ERR=false

${CAT} > ${QMGRCONF} <<EOF
smtpc_conf_mail:@other.tld	source=bind1;
smtpc_conf_mail:sender@other.tld	source=bind0;
EOF
rm -f ${QMGRCONFMAP}
${MM} -t'	' -F ${QMGRCONFMAP} < ${QMGRCONF}

N=8
if ${SHELL} ${SD}/smtpcnf.sh -F ${SRVPORT} -E $N -X $N -A $N -Z "smtpc2" -c "-fsender@other.tld -Ruser@some.dom -fs1@other.tld -m $N -s $N" -d 5 -Y "-d 8"
then
  if grep "connection from $H2" ${SRVL} >/dev/null
  then
    if grep "WARN, .*ip=$HI, bind" ${SCL} >/dev/null
    then
      ERR=true
      $VERBOSE && echo "$0: found bind warning in $SCL"
    fi
  else
    ERR=true
    $VERBOSE && echo "$0: missing connection from $H2"
  fi
else
  ERR=true
fi
if ${ERR}
then
   echo "$0: test ${TEST} failed"
   ERRS=`expr ${ERRS} + 1 `
   test X"${MTA_STOPONERROR}" != X && exit 1
fi
else
  ${VERBOSE} && echo "$0: test ${TEST} SKIPPED"
fi


# ----------------------------------------
# test: send many mails
TEST=6
if echo " ${SKIP} X" | ${GREP} -v " ${TEST} " > /dev/null
then
ERR=false

${CAT} > ${QMGRCONF} <<EOF
smtpc_conf_mail:@other.tld	source=bind1;
smtpc_conf_mail:sender@other.tld	source=bind0;
EOF
rm -f ${QMGRCONFMAP}
${MM} -t'	' -F ${QMGRCONFMAP} < ${QMGRCONF}

N=20
if ${SHELL} ${SD}/smtpcnf.sh -F ${SRVPORT} -E $N -X $N -A $N -Z "smtpc2" -c "-fsender@other.tld -Ruser@some.dom -fs1@other.tld -m $N -s $N -t 5" -d 5 -Y "-d 8"
then
  if grep "connection from $H2" ${SRVL} >/dev/null
  then
    if grep "WARN, .*ip=$HI, bind" ${SCL} >/dev/null
    then
      ERR=true
      $VERBOSE && echo "$0: found bind warning in $SCL"
    fi
  else
    ERR=true
    $VERBOSE && echo "$0: missing connection from $H2"
  fi
else
  ERR=true
fi
if ${ERR}
then
   echo "$0: test ${TEST} failed"
   ERRS=`expr ${ERRS} + 1 `
   test X"${MTA_STOPONERROR}" != X && exit 1
fi
else
  ${VERBOSE} && echo "$0: test ${TEST} SKIPPED"
fi



# ----------------------------------------
# test: send mails, use invalid bind address
# [should be last in list, uses a different .conf!]
TEST=`expr $TEST + 1 `
if echo " ${SKIP} X" | ${GREP} -v " ${TEST} " > /dev/null
then
ERR=false

# really invalid?
HI=1.252.253.254

${CAT} > ${SMXCNF} <<EOF
smtpc { Log_Level = 13; remote_port=${SNKPORT}; wait_for_server=4;
}
smtps {
	Log_Level = 12;
	daemon_address = localhost:${SRVPORT};
	wait_for_server=4;
}
qmgr { Log_Level = 12; wait_for_server=4; wait_for_client=4;
  smtpc {
	source bind0 { ips = { $HI } }
  }
}
smar { Log_Level = 12; nameserver = ${NS}; }
EOF

${CAT} > ${QMGRCONF} <<EOF
smtpc_conf_mail:@some.dom	source=bind0;
EOF
rm -f ${QMGRCONFMAP}
${MM} -t'	' -F ${QMGRCONFMAP} < ${QMGRCONF}

if ${SHELL} ${SD}/smtpcnf.sh -F ${SRVPORT} -Ssender@some.dom -Ruser@some.dom -E 1 -X 1 -A 1 -Z "smtpc2" -d 5 -Y "-d 8"
then
  if grep "connection from $HI" ${SRVL} >/dev/null
  then
    ERR=true
    $VERBOSE && echo "$0: found connection from $HI"
  else
    if grep "WARN, .*ip=$HI, bind" ${SCL} >/dev/null
    then
      :
    else
      ERR=true
      $VERBOSE && echo "$0: missing bind warning in $SCL"
    fi
  fi
else
  ERR=true
fi
if ${ERR}
then
   echo "$0: test ${TEST} failed"
   ERRS=`expr ${ERRS} + 1 `
   test X"${MTA_STOPONERROR}" != X && exit 1
fi
else
  ${VERBOSE} && echo "$0: test ${TEST} SKIPPED"
fi

# ----------------------------------------
# end
if test ${ERRS} -eq 0
then
  exit 0
else
  echo "$0: ${ERRS} error(s)"
  exit 1
fi
