#!/bin/sh
# $Id: t-mts-lum-0.sh,v 1.4 2010/02/16 17:01:38 ca Exp $
# Copyright (c) 2005 Sendmail, Inc. and its suppliers.
#	All rights reserved.
#
# By using this file, you agree to the terms and conditions set
# forth in the LICENSE file which can be found at the top level of
# the sendmail distribution.
#
# ----------------------------------------
# test acception/rejection of MAIL with local user map
# uses smtpcnf.sh
# ----------------------------------------
#

SD=`dirname $0`
if test "${SD}" = "."
then
   SD=../chkmts
fi
. ${SD}/common.sh

LUM="lu.db"

if test X"`uname`" != "XOSF1"
then
while getopts s:V FLAG
do
  case "${FLAG}" in
    s) SKIP="${SKIP} ${OPTARG}";;
    V) VERBOSE=true;;
  esac
done
shift `expr ${OPTIND} - 1`
fi

# cd to working directory
cd ../qmgr || exit 1

# remove maps
rm -f ${ALIMAP}
rm -f ${ACCMAP}

# create local user map
rm -f ${LUM}
${MM} -F ${LUM} <<EOF
lu1:	ok
EOF
if test $? -ne 0
then
  echo "$0: ${MM} failed"
  exit 1
fi

# create mailertable (save old version?)
${CAT} > ${MT} <<EOF
local.dom	lmtp:
some.dom	[127.0.0.1]
EOF

# ----------------------------------------
${CAT} > ${SMXCNF} <<EOF
qmgr { Log_Level = 12; wait_for_server=4; wait_for_client=4;
}
smar {
 #map password { type = passwd; }
 #local_user_map { name=password;}
 map lum { type=hash; file=${LUM}; }
 local_user_map { name=lum;}
Log_Level = 12;
nameserver = ${NS};
DNS_timeout = 5;
}
smtpc { Log_Level = 12; remote_port=${SNKPORT}; wait_for_server=4; }
smtps {
Log_Level = 14;
daemon_address = localhost:${SRVPORT};
wait_for_server=4;
}
EOF

# ----------------------------------------
# test: send mail with local address which is not in lum, hence sender is
# rejected by smtps.

TEST=1
if echo " ${SKIP} X" | ${GREP} -v " ${TEST} " > /dev/null
then
ERR=false

if ${SHELL} ${SD}/smtpcnf.sh -3 lmtpsock -Stestsender${TEST}@local.dom -Rlu1@local.dom -E 0 -X 0 -A 0
then
  if test -s ${B1}
  then
    ERR=true
  else
    :
  fi
else
  ERR=true
fi
if ${ERR}
then
   echo "$0: test ${TEST} failed"
   ERRS=`expr ${ERRS} + 1 `
   test X"${MTA_STOPONERROR}" != X && exit 1
fi
rm -f ${B1}
else
  ${VERBOSE} && echo "$0: test ${TEST} SKIPPED"
fi


# ----------------------------------------
# test: send mail with local recipient which is in lum, hence mail is
# accepted by smtps.

TEST=2
if echo " ${SKIP} X" | ${GREP} -v " ${TEST} " > /dev/null
then
ERR=false

if ${SHELL} ${SD}/smtpcnf.sh -3 lmtpsock -Ssender@some.dom -Rlu1@local.dom -E 0 -X 0 -A 1
then
  if test -s ${B1}
  then
    :
  else
    ERR=true
  fi
else
  ERR=true
fi
if ${ERR}
then
   echo "$0: test ${TEST} failed"
   ERRS=`expr ${ERRS} + 1 `
   test X"${MTA_STOPONERROR}" != X && exit 1
fi
rm -f ${B1}
else
  ${VERBOSE} && echo "$0: test ${TEST} SKIPPED"
fi

# ----------------------------------------
# test: send mail with local recipient which is not in lum, hence recipient is
# rejected by smtps.

TEST=3
if echo " ${SKIP} X" | ${GREP} -v " ${TEST} " > /dev/null
then
ERR=false

if ${SHELL} ${SD}/smtpcnf.sh -3 lmtpsock -Stestsender${TEST}@some.dom -Rno${TEST}@local.dom -E 0 -X 0 -A 0
then
  if test -s ${B1}
  then
    ERR=true
  else
    :
  fi
else
  ERR=true
fi
if ${ERR}
then
   echo "$0: test ${TEST} failed"
   ERRS=`expr ${ERRS} + 1 `
   test X"${MTA_STOPONERROR}" != X && exit 1
fi
rm -f ${B1}
else
  ${VERBOSE} && echo "$0: test ${TEST} SKIPPED"
fi


rm -f ${MTMAP} ${ALIMAP} ${LUM}
# ----------------------------------------
# end
if test ${ERRS} -eq 0
then
  exit 0
else
  echo "$0: ${ERRS} error(s)"
  exit 1
fi
