#!/bin/sh
# $Id: t-tls-cno-1.sh,v 1.3 2013/10/20 20:03:44 ca Exp $
#
# ----------------------------------------
# test turning off TLS in smtpc via session_feature; check delivered mail
# creates two MTA (one in chkmts, one in qmgr) to use STARTTLS
# run smtp1.sh with different parameters
# ----------------------------------------
#

SD=`dirname $0`
THISDIR=`pwd`
if test "${SD}" = "."
then
   SD=../chkmts
fi
. ${SD}/common.sh

chkcto smtps MTA_USE_TLS
chkcto smtpc MTA_USE_TLS
chkcto smtpc SC_SESS_FEATS

OUT="t-tls-cno-0.out"
PID2=pids2

if test X"`uname`" != "XOSF1"
then
while getopts s:V FLAG
do
  case "${FLAG}" in
    s) SKIP="${SKIP} ${OPTARG}";;
    V) VERBOSE=true;;
  esac
done
shift `expr ${OPTIND} - 1`
fi

killit()
{
cd ${THISDIR}
if test -s ${PID2}
then
  # stop MTA components
  for i in `${CAT} ${PID2}`
  do
    kill ${i}
  done
fi
# ----------------
# thanks kids for not following the POSIX standard...
if test X"`uname`" = "XLinux"
then
  killall qmgr smar
fi
exit 1
}

# cd to working directory for alias file!
cd ../qmgr || exit 1
setuptls0

# ----------------------------------------
# need two SMTP servers:
# test client [smtp1.sh] -> SMTP server 1 (SRVPORT)
# -> QMGR 1 -> SMTP client 1 (SNKPORT)
# -> SMTP server 2 [smtp1.sh] (SRVPORT2=SNKPORT)
# -> QMGR 2 [smtp1.sh] -> SMTP client 2 [smtp1.sh]
# -> lmtp sink [smtp1.sh]
# problem: test client in smtp1.sh will use SRVPORT2
# hence the port must be passed explicitly via -F

cd ${THISDIR} || exit 1
setuptls1

# ----------------------------------------
# start first MTA (chkmts, it will send the e-mails)
#
CONF2=smx2.conf
${CAT} > ${CONF2} <<EOF
qmgr { Log_Level = 12; wait_for_server=4; wait_for_client=4;
	smtpc {flags = lookup_session_conf;}
	start_action = wait; user = ${USER};
	restart_dependencies = { smtps,smtpc,smar, };
	path = "../qmgr/qmgr"; arguments = "qmgr -f ${CONF2}";
}
smar { Log_Level = 12;
	start_action = wait; user = ${USER};
	restart_dependencies = { smtps,qmgr };
	path = "../smar/smar"; arguments = "smar -f ${CONF2}";
}
smtpc { Log_Level = 12; remote_port=${SNKPORT}; wait_for_server=4;
  flags = talk_to_myself ; # don't check "talking to myself"
  session_features notls { flags = { -starttls }}
	start_action = wait; user = ${USER};
	path = "../smtpc/smtpc"; arguments = "smtpc -f ${CONF2}";
}
smtps {
Log_Level = 12;
daemon_address = localhost:${SRVPORT};
wait_for_server=4;
	listen_socket { type=inet; port=${SRVPORT}; }
	start_action = wait; user = ${USER};
	path = ../smtps/smtps;
	arguments = "smtps -f ${CONF2} ${MTA_SERVER_OPTIONS}";
}
EOF

${CAT} > ${QMGRCONF} <<EOF
smtpc_session_conf:127.0.0.1	session_feature=notls;
EOF
rm -f ${QMGRCONFMAP}
${MM} -t'	' -F ${QMGRCONFMAP} < ${QMGRCONF}

# version of output files
V=1
# path to programs
P=..
rm -f ${PID2}
trap killit 2 15
# go for it
${P}/smar/smar -f ${CONF2} > a${V}.log 2>&1 &
echo $! >> ${PID2}
${P}/qmgr/qmgr -f ${CONF2} > q${V}.log 2>&1 &
echo $! >> ${PID2}
${P}/smtpc/smtpc -f ${CONF2} > c${V}.log 2>&1 &
echo $! >> ${PID2}
${P}/smtps/smtps -f ${CONF2} > s${V}.log 2>&1 &
echo $! >> ${PID2}

# check that the MTA is running
export P
${CHKD}/mta-running.sh ${SRVPORT} || killit

export MTA_SNKPORT MTA_SRVPORT

# ----------------------------------------
# test: send mail to local address, check Received: header for TLS

TEST=1
if echo " ${SKIP} X" | ${GREP} -v " ${TEST} " > /dev/null
then
ERR=false

cd ../qmgr
if ${SHELL} ${SD}/smtp1.sh -F ${SRVPORT} -3 lmtpsock -Ssender@other.dom -Ruser@local.dom -E 0 -X 0 -A 1
then
  if test -s ${B1}
  then
    if ${GREP} 'TLS' ${B1} >/dev/null
    then
      ${VERBOSE} && echo "$0: test ${TEST}: TLS found in ${B1}"
      ERR=true
    fi
  else
    ${VERBOSE} && echo "$0: test ${TEST}: ${B1} not found"
    ERR=true
  fi
else
  ERR=true
fi
if ${ERR}
then
   echo "$0: test ${TEST} failed"
   ERRS=`expr ${ERRS} + 1 `
   test X"${MTA_STOPONERROR}" != X && exit 1
fi
rm -f ${B1}
else
  ${VERBOSE} && echo "$0: test ${TEST} SKIPPED"
fi
rm -f ${MTMAP} ${ALIMAP}


# ----------------------------------------
cd ${THISDIR}
rm -f ${MTMAP} ${ALIMAP}
if test -s ${PID2}
then
  # stop MTA components
  for i in `${CAT} ${PID2}`
  do
    kill ${i}
  done
fi
# ----------------
# thanks kids for not following the POSIX standard...
if test X"`uname`" = "XLinux"
then
  killall qmgr smar
fi

testend
