#!/bin/sh
# $Id: t-tls-reqrc-0.sh,v 1.3 2014/06/03 03:51:21 ca Exp $
#
# Copyright (c) 2014 Claus Assmann
#
# By using this file, you agree to the terms and conditions set
# forth in the license/LICENSE.3C file which can be found at the
# top level of this source code distribution.
#
# ----------------------------------------
# smptc TLS requirement
# uses smtps2 with TLS
# run smtpcnf.sh with different parameters
# ----------------------------------------
#

SD=`dirname $0`
if test "${SD}" = "."
then
   SD=../chkmts
fi
. ${SD}/common.sh

SINKPRG=${STE}/smtps3
chkcto smtpc MTA_USE_TLS
sinkhastls

if test X"`uname`" != "XOSF1"
then
while getopts s:V FLAG
do
  case "${FLAG}" in
    s) SKIP="${SKIP} ${OPTARG}";;
    V) VERBOSE=true;;
  esac
done
shift `expr ${OPTIND} - 1`
fi

# cd to working directory
cd ../qmgr || exit 1

SRCPRG=${STE}/smtpc2
SENDER=sender@local.dom
RCPT=user@local.host

rm -f ${ALIMAP} ${MTMAP} ${ACCMAP}
# create aliases
${CAT} > ${ALI} <<EOF
abuse:	local:
postmaster:	local:
user:	local:
sender: local:
EOF
# create map
${MM} < ${ALI}

# ----------------------------------------
# test: smtpc_rcpt_conf not fullfilled
TEST=1
if echo "${SKIP} X" | ${GREP} -v " ${TEST} " > /dev/null
then
ERR=false

${CAT} > ${SMXCNF} <<EOF
qmgr { Log_Level = 12; wait_for_server=4; wait_for_client=4;
  smtpc { flags = {lookup_rcpt_conf,lookup_session_conf,} }
}
smar { Log_Level = 12; nameserver = ${NS}; }
smtpc { Log_Level = 12; remote_port=${SNKPORT}; wait_for_server=4; }
smtps { Log_Level = 14; daemon_address = localhost:${SRVPORT};
  wait_for_server=4;
}
EOF

rm -f ${EDBROK}
cat > ${EDBROK} <<EOF
got transaction
	mail=<sender@local.dom>
	rcpts_tot=1
	rcpts_left=1
	rcpts_temp=1
	rcpts_perm=0
	state=421
	aqt_rcpts_ar=0
got recipient
	rcpt=<user@local.host>
	aqr_rcpt_idx=0
	aqr_da_idx=0
	aqr_status=421
EOF
crtacc "smtpc_rcpt_conf:${RCPT}	tls_requirements {cert_issuer=\"/C=US/ST=CA/L=B/OU=MTA/CN=CA/$EM=CA@example.org\";}"
crtqcnf "smtpc_session_conf:127.	tls_requirements { flags={verified}}"
if ${SHELL} ${SD}/smtpcnf.sh -O ${EDBROK} -r -Y "-@S -d4" -E 0 -A 1 -X 1 -S${SENDER} -R${RCPT} -s 12
then
  :
else
  ERR=true
fi
if ${ERR}
then
   echo "$0: test ${TEST} failed"
   ERRS=`expr ${ERRS} + 1 `
   test X"${MTA_STOPONERROR}" != X && exit 1
fi
fi

# ----------------------------------------
# test: smtpc_rcpt_conf map lookup problem
TEST=2
if echo "${SKIP} X" | ${GREP} -v " ${TEST} " > /dev/null
then
ERR=false

${CAT} > ${SMXCNF} <<EOF
qmgr { Log_Level = 12; wait_for_server=4; wait_for_client=4;
  smtpc { flags = {lookup_rcpt_conf,lookup_session_conf,} }
  retry_delay_min = 2; retry_delay_max = 4;
}
smar { Log_Level = 12; nameserver = ${NS}; }
smtpc { Log_Level = 12; remote_port=${SNKPORT}; wait_for_server=4; }
smtps {
  Log_Level = 14; daemon_address = localhost:${SRVPORT};
  wait_for_server=4;
}
EOF

rm -f ${EDBROK}
cat > ${EDBROK} <<EOF
got transaction
	mail=<sender@local.dom>
	rcpts_tot=1
	rcpts_left=1
	rcpts_temp=1
	rcpts_perm=0
	state=483
	aqt_rcpts_ar=0
got recipient
	rcpt=<user@local.host>
	aqr_rcpt_idx=0
	aqr_da_idx=0
	aqr_status=483
EOF

rm -f ${ACCMAP} ${ACC}
printf "smtpc_rcpt_conf:${RCPT}\ttls_requirements {cert_issuer=\"/C=US/ST=CA/L=B/OU=MTA/CN=CA/$EM=CA@example.org\";" > ${ACC}
# XREF: MAXCONFLEN
printf "%4096s\n" "}" >> ${ACC}

${MM} -t'	' -F ${ACCMAP} < ${ACC}
crtqcnf "smtpc_session_conf:127.	tls_requirements { flags={verified}}"
if ${SHELL} ${SD}/smtpcnf.sh -O ${EDBROK} -r -Y "-@S -d4" -E 0 -A 1 -X 0 -S${SENDER} -R${RCPT} -s 12
then
  :
else
  ERR=true
fi
if ${ERR}
then
   echo "$0: test ${TEST} failed"
   ERRS=`expr ${ERRS} + 1 `
   test X"${MTA_STOPONERROR}" != X && exit 1
fi
fi


# ----------------------------------------
rm -f ${MTMAP} ${ALIMAP} ${QMGRCONFMAP} ${QMGRCONF} ${EDBROK}

testend
