#!/bin/sh
# $Id: t-tls-reqrc-1.sh,v 1.3 2014/06/03 02:46:14 ca Exp $
#
# Copyright (c) 2014 Claus Assmann
#
# By using this file, you agree to the terms and conditions set
# forth in the license/LICENSE.3C file which can be found at the
# top level of this source code distribution.
#
# ----------------------------------------
# smptc TLS requirement: map lookup problems
# uses smtps2 with TLS
# run smtpcnf.sh with different parameters
# ----------------------------------------
#

SD=`dirname $0`
if test "${SD}" = "."
then
   SD=../chkmts
fi
. ${SD}/common.sh

SINKPRG=${STE}/smtps3
chkcto smtpc MTA_USE_TLS
chkcto smar MTA_TEST_MAP
sinkhastls

if test X"`uname`" != "XOSF1"
then
while getopts s:V FLAG
do
  case "${FLAG}" in
    s) SKIP="${SKIP} ${OPTARG}";;
    V) VERBOSE=true;;
  esac
done
shift `expr ${OPTIND} - 1`
fi

# cd to working directory
cd ../qmgr || exit 1

SRCPRG=${STE}/smtpc2
SENDER=sender@local.dom
RCPT=m4@local.host

${CAT} > ${SMXCNF} <<EOF
qmgr { Log_Level = 12; wait_for_server=4; wait_for_client=4;
  retry_delay_min = 2; retry_delay_max = 120;
  smtpc { flags = {lookup_rcpt_conf,lookup_session_conf,} }
}
smar { Log_Level = 12; nameserver = ${NS};
  map acc { type=tst; }
  access_map { name=acc;}
}
smtpc { Log_Level = 12; remote_port=${SNKPORT}; wait_for_server=4;
}
smtps {
Log_Level = 14;
daemon_address = localhost:${SRVPORT};
wait_for_server=4;
flags = {access};
}
EOF

rm -f ${ALIMAP} ${MTMAP} ${ACCMAP}
# create aliases
${CAT} > ${ALI} <<EOF
abuse:	local:
postmaster:	local:
user:	local:
sender: local:
EOF
# create map
${MM} < ${ALI}

# ----------------------------------------
# test: temp lookup fail of requirement, hence mail stays in queue
TEST=1
if echo "${SKIP} X" | ${GREP} -v " ${TEST} " > /dev/null
then
ERR=false
rm -f ${EDBROK}
cat > ${EDBROK} <<EOF
got transaction
	mail=<sender@local.dom>
	rcpts_tot=1
	rcpts_left=1
	rcpts_temp=1
	rcpts_perm=0
	state=483
	aqt_rcpts_ar=0
got recipient
	rcpt=<m4@local.host>
	aqr_rcpt_idx=0
	aqr_da_idx=0
	aqr_status=483
EOF
crtacc "smtpc_rcpt_conf:${RCPT}	tls_requirements {cert_issuer=\"/C=US/ST=CA/L=B/OU=MTA/CN=CA/$EM=CA@example.org\"; requirements_violation=permfail;}"
crtqcnf "smtpc_session_conf:127.	tls_requirements { flags={verified}}"
if ${SHELL} ${SD}/smtpcnf.sh -O ${EDBROK} -r -Y "-@S -d4" -E 0 -A 1 -X 0 -S${SENDER} -R${RCPT} -s 4
then
  :
else
  ERR=true
fi
if ${ERR}
then
   echo "$0: test ${TEST} failed"
   ERRS=`expr ${ERRS} + 1 `
   test X"${MTA_STOPONERROR}" != X && exit 1
fi
fi

# ----------------------------------------
# test: temp lookup fail of requirement, then lookup returns "no match"
TEST=2
if echo "${SKIP} X" | ${GREP} -v " ${TEST} " > /dev/null
then
RCPT=mn@local.host
ERR=false
rm -f ${EDBROK}
crtacc "smtpc_rcpt_conf:${RCPT}	tls_requirements {cert_issuer=\"/C=US/ST=CA/L=B/OU=MTA/CN=CA/$EM=CA@example.org\"; requirements_violation=permfail;}"
crtqcnf "smtpc_session_conf:127.	tls_requirements { flags={verified}}"
if ${SHELL} ${SD}/smtpcnf.sh -d 9 -r -Y "-@S -d4" -E 1 -A 1 -X 1 -S${SENDER} -R${RCPT} -s 8
then
  :
else
  ERR=true
fi
if ${ERR}
then
   echo "$0: test ${TEST} failed"
   ERRS=`expr ${ERRS} + 1 `
   test X"${MTA_STOPONERROR}" != X && exit 1
fi
fi


# ----------------------------------------
rm -f ${MTMAP} ${ALIMAP} ${QMGRCONFMAP} ${QMGRCONF} ${EDBROK}

testend
