/*
 * Copyright (c) 2004-2006, 2011 Sendmail, Inc. and its suppliers.
 *      All rights reserved.
 *
 * By using this file, you agree to the terms and conditions set
 * forth in the LICENSE file which can be found at the top level of
 * the sendmail distribution.
 *
 *	$Id: qmgrcnfdef.h,v 1.116 2013/12/07 03:06:15 ca Exp $
 */

#ifndef SM_QMGRCNFDEF_H
#define SM_QMGRCNFDEF_H 1

#include "sm/generic.h"
#include "sm/common.h"
#include "sm/magic.h"
#include "sm/cnf.h"
#include "sm/edbdef.h"
#include "sm/qmgrdef.h"
#include "sm/qmgrcnf.h"
#include "sm/mcpcnfs.h"
#include "sm/mta.h"
#include "sm/sm-conf-time.h"
#include "sm/sm-conf-byte.h"
#include "sm/mapcnfs.h"

/*
**  Note: the defaults in here are ignored as long as SM_CONF_FLAG_KEEP_DEFAULT
**  is set, which is necessary to avoid zero'ing out the structure.
*/

#if SM_QMGRCNFDEF
#define EXTERN
#else /* SM_QMGRCNFDEF */
#define EXTERN extern
#endif /* SM_QMGRCNFDEF */

#define QMGR_CNF_AQ_SIZE	"AQ_entries_max"
#define QMGR_CNF_OCC_SIZE	"OCC_entries_max"
#define QMGR_CNF_IBDB_SIZE	"size"
#define QMGR_CNF_IQDB_CACHE_SIZE	"cache_entries_max"
#define QMGR_CNF_IQDB_HASH_TABLE_SIZE	"hash_table_entries"
#define QMGR_CNF_EDBC_HASH_TABLE_SIZE	QMGR_CNF_IQDB_HASH_TABLE_SIZE
#define QMGR_CNF_DISK_SPACE_MIN	"disk_space_min"
#define QMGR_CNF_OK_DISK_SPACE	"disk_space_ok"
#define QMGR_CNF_OPEN_TAS_MAX	"open_TAs_max"

#define QMGR_CNF_SC_INITIAL_CONNS	"initial_connections"
#define QMGR_CNF_SC_CONNS_MAX		"connections_max"
#define QMGR_CNF_SS_OPEN_CONNS_MAX	"connections_max"
#define QMGR_CNF_SS_CONN_RATE_MAX	"connection_rate_max"
#define QMGR_CNF_SS_CONNCTL_SIZE	"connection_control_hash_table_size"

#define QMGR_CNF_SC_TAS_SE	"transactions_per_session_max"

#define QMGR_CNF_SC_SMTP_RCPTS_TA	"smtp_rcpts_per_transaction_max"
#define QMGR_CNF_SC_LMTP_RCPTS_TA	"lmtp_rcpts_per_transaction_max"

#define QMGR_CNF_ERRORS_PER_DSN_MAX	"errors_per_DSN_max"
#define QMGR_CNF_FLAGS	"flags"
#define QMGR_CNF_IBDB_COMMIT_DELAY_MAX	"commit_delay_max"
#define QMGR_CNF_DELAY_RETRIES_MIN	"retry_delay_min"
#define QMGR_CNF_DELAY_RETRIES_MAX	"retry_delay_max"
#define QMGR_CNF_TMOUT_QUEUERETURN	"queue_return_timeout"
#define QMGR_CNF_TMOUT_QUEUEDELAY	"queue_delay_timeout"
#define QMGR_CNF_TMOUT_AR	"SMAR_timeout"
#define QMGR_CNF_TMOUT_DELIVERY	"delivery_timeout"
#define QMGR_CNF_TMOUT_SCHED	"scheduler_timeout"
#define QMGR_CNF_DSN_DELAY_MAX	"merge_delay_max"
#define QMGR_CNF_WAIT_FOR_SERVER	"wait_for_server"
#define QMGR_CNF_WAIT_FOR_CLIENT	"wait_for_client"
#define QMGR_CNF_LOG_LEVEL	"log_level"
#define QMGR_CNF_DEBUG_LEVEL	"debug_level"
#define QMGR_CNF_TESTS	"tests"
#define QMGR_CNF_THR_MIN	"threads_min"
#define QMGR_CNF_THR_MAX	"threads_max"
#define QMGR_CNF_FD_MAX		"fds_max"


#define QMEDB_CNF_BASEDIR	"base_directory"
#define QMEDB_CNF_LOGDIR	"log_directory"
#define QMEDB_CNF_PAGESIZE	"page_size"
#define QMEDB_CNF_CACHESIZE	"cache_size"
#define QMEDB_CNF_CHKP_KB	"KBytes_written_for_checkpointing"
#define QMEDB_CNF_CHKP_DELAY	"delay_between_2_checkpoints"
#define QMEDB_CNF_FATAL_RCVR	"fatal_recovery"
#define QMEDB_CNF_RM_LOGFILES	"remove_unused_logfiles"

#define QMGR_CNF_SC_LKP_RCPT_CONF "lookup_rcpt_conf"
#define QMGR_CNF_SC_LKP_SE_CONF "lookup_session_conf"

EXTERN
sm_conf_definition_T sc_cl_defs[]
#if SM_QMGRCNFDEF
= {
{ SM_CONF_DEF_MAGIC, "name",		sm_conf_type_section_title,
	offsetof(sc_cl_T, sccl_name),	0,
	NULL,	SM_CONF_FLAG_STRICTLY_REQUIRED,
	NULL, NULL, NULL, NULL
	SM_LC_NO_ISSET	SM_LC_SET_MAGIC(0) },

{ SM_CONF_DEF_MAGIC, QMGR_CNF_SC_TAS_SE,		sm_conf_type_u32,
	offsetof(sc_cl_T, sccl_max_tas_se),	sizeof(uint32_t),
	SM_XSTR(DA_TAS_SE_MAX), SM_CONF_FLAG_DEFAULT_FROM_ENVIRONMENT,
	NULL, NULL,	NULL,
	"maximum number of transactions per session"
	SM_LC_NO_ISSET	SM_LC_SET_MAGIC(0) },

{ SM_CONF_DEF_MAGIC, QMGR_CNF_SC_INITIAL_CONNS,		sm_conf_type_u32,
	offsetof(sc_cl_T, sccl_init_conc_conn),	sizeof(uint32_t),
	NULL, SM_CONF_FLAG_DEFAULT_FROM_ENVIRONMENT,
	NULL, NULL,	NULL,
	"initial number of outgoing connections"
	SM_LC_NO_ISSET	SM_LC_SET_MAGIC(0) },

{ SM_CONF_DEF_MAGIC, QMGR_CNF_SC_CONNS_MAX,		sm_conf_type_u32,
	offsetof(sc_cl_T, sccl_max_conc_conn),	sizeof(uint32_t),
	NULL, SM_CONF_FLAG_DEFAULT_FROM_ENVIRONMENT,
	NULL, NULL,	NULL,
	"maximum number of outgoing connections"
	SM_LC_NO_ISSET	SM_LC_SET_MAGIC(0) },

#if OCC_RATE
{ SM_CONF_DEF_MAGIC, "message_rate_max",		sm_conf_type_u32,
	offsetof(sc_cl_T, sccl_max_msg_rate),	sizeof(uint32_t),
	NULL, SM_CONF_FLAG_DEFAULT_FROM_ENVIRONMENT,
	NULL, NULL,	NULL,
	"maximum number of messages per minute"
	SM_LC_NO_ISSET	SM_LC_SET_MAGIC(0) },
#endif

CONF_SENTINEL
}
#endif /* SM_QMGRCNFDEF */
;

EXTERN
sm_conf_definition_T sc_cl_definitions[]
#if SM_QMGRCNFDEF
= {
{ SM_CONF_DEF_MAGIC, "",	sm_conf_type_section,
	offsetof(qmgr_cnf_T, q_cnf_sc_cls.sccls_cl), sizeof(sc_cl_T),
	NULL, SM_CONF_FLAG_DEFAULT_FROM_ENVIRONMENT,
	sc_cl_defs,
	NULL, NULL, "connection limits" },

{ SM_CONF_DEF_MAGIC, "n",	sm_conf_type_array_n,
	offsetof(qmgr_cnf_T, q_cnf_sc_cls.sccls_n), sizeof(uint),
	NULL, 0,
	sc_cl_defs,
	NULL, NULL,
	"number of connection limits (implicit)" },

CONF_SENTINEL
}
#endif /* SM_QMGRCNFDEF */
;


EXTERN
sm_conf_definition_T const
qmgr_flag_names[]
#if SM_QMGRCNFDEF
=
{
{ SM_CONF_DEF_MAGIC, "reuse_connection",	sm_conf_type_choice_value,
	QCNF_FL_SE_REUSE,
	0, NULL, 0, NULL, NULL, NULL,
	"reuse existing connection if possible"
	SM_LC_NO_ISSET	SM_LC_SET_MAGIC(0) },

CONF_SENTINEL
}
#endif /* SM_QMGRCNFDEF */
;

EXTERN
sm_conf_definition_T const
qmgr_dsn_flag_names[]
#if SM_QMGRCNFDEF
=
{
{ SM_CONF_DEF_MAGIC, "header_only",	sm_conf_type_choice_value,
	QCNF_FL_B_HDR,
	0, NULL, 0, NULL, NULL, NULL,
	"send only headers in a DSN"
	SM_LC_NO_ISSET	SM_LC_SET_MAGIC(0) },
{ SM_CONF_DEF_MAGIC, "MIME_format",	sm_conf_type_choice_value,
	QCNF_FL_B_DSN_MIME,
	0, NULL, 0, NULL, NULL, NULL,
	"send DSN in MIME format"
	SM_LC_NO_ISSET	SM_LC_SET_MAGIC(0) },
{ SM_CONF_DEF_MAGIC, "rfc_format",	sm_conf_type_choice_value,
	QCNF_FL_B_DSN_RFC,
	0, NULL, 0, NULL, NULL, NULL,
	"send DSN in RFC 3464 format"
	SM_LC_NO_ISSET	SM_LC_SET_MAGIC(0) },

CONF_SENTINEL
}
#endif /* SM_QMGRCNFDEF */
;


EXTERN
sm_conf_definition_T
qmgr_dsn_defs[]
#if SM_QMGRCNFDEF
=
{
{ SM_CONF_DEF_MAGIC, QMGR_CNF_ERRORS_PER_DSN_MAX,	sm_conf_type_u32,
	offsetof(qmgr_cnf_T, q_cnf_dsn_rcpts_max),	sizeof(uint),
	SM_XSTR(AQR_DSN_RCPTS_MAX),	0,	NULL,	NULL,	NULL,
	"maximum number of error messages (failed recipients) in a bounce (DSN)"
	SM_LC_NO_ISSET	SM_LC_SET_MAGIC(0) },

{ SM_CONF_DEF_MAGIC, QMGR_CNF_DSN_DELAY_MAX,		sm_conf_type_u32,
	offsetof(qmgr_cnf_T, q_cnf_t_sched_dsn),	sizeof(uint),
	SM_XSTR(QM_TMO_SCHED_DSN),	0,	sm_conf_time_suffixes,
	NULL,	NULL,
	"maximum time to wait for merging multiple DSNs into one"
	SM_LC_NO_ISSET	SM_LC_SET_MAGIC(0) },

{ SM_CONF_DEF_MAGIC, QMGR_CNF_FLAGS,			sm_conf_type_choice,
	offsetof(qmgr_cnf_T, q_cnf_flags),		sizeof(uint32_t),
	NULL,	SM_CONF_FLAG_MULTIPLE|SM_CONF_FLAG_OR|SM_CONF_FLAG_PLURAL,
	qmgr_dsn_flag_names,	NULL,	NULL,
	"flags for DSN handling"
	SM_LC_NO_ISSET	SM_LC_SET_MAGIC(0) },

CONF_SENTINEL
}
#endif /* SM_QMGRCNFDEF */
;


EXTERN
sm_conf_definition_T const
qmgr_edb_flag_names[]
#if SM_QMGRCNFDEF
=
{
{ SM_CONF_DEF_MAGIC, QMEDB_CNF_FATAL_RCVR,	sm_conf_type_choice_value,
	EDB_OPEN_RECOVER_FATAL,
	0, NULL, 0, NULL, NULL, NULL, "perform fatal recovery"
	SM_LC_NO_ISSET	SM_LC_SET_MAGIC(0) },

{ SM_CONF_DEF_MAGIC, QMEDB_CNF_RM_LOGFILES,	sm_conf_type_choice_value,
	EDB_LOG_AUTOREMOVE,
	0, NULL, 0, NULL, NULL, NULL,
	"automatically remove unused logfiles"
	SM_LC_NO_ISSET	SM_LC_SET_MAGIC(0) },

CONF_SENTINEL
}
#endif /* SM_QMGRCNFDEF */
;


EXTERN
sm_conf_definition_T const
edbc_ht_constraints[]
#if SM_QMGRCNFDEF
=
{
{ SM_CONF_DEF_MAGIC, "", sm_conf_type_u32_minimum,
	128,	0,
	NULL,	0,	NULL,	NULL,	NULL,
	""
	SM_LC_NO_ISSET	SM_LC_SET_MAGIC(0) },
{ SM_CONF_DEF_MAGIC, "", sm_conf_type_u32_maximum,
	64 * 1024 * 1024,	0,
	NULL,	0,	NULL,	NULL,	NULL,
	""
	SM_LC_NO_ISSET	SM_LC_SET_MAGIC(0) },
    
CONF_SENTINEL
}
#endif /* SM_QMGRCNFDEF */
;

EXTERN
sm_conf_definition_T
qmgr_edb_defs[]
#if SM_QMGRCNFDEF
=
{
{ SM_CONF_DEF_MAGIC, QMEDB_CNF_BASEDIR,	sm_conf_type_string,
	offsetof(qmgr_cnf_T, q_cnf_edb_cnf.edbcnf_basedir),	0,
	NULL,	0,	NULL,	NULL,	NULL,
	"Home directory for DEFEDB"
	SM_LC_NO_ISSET	SM_LC_SET_MAGIC(0) },

{ SM_CONF_DEF_MAGIC, QMEDB_CNF_LOGDIR,	sm_conf_type_string,
	offsetof(qmgr_cnf_T, q_cnf_edb_cnf.edbcnf_logdir),	0,
	NULL,	0,	NULL,	NULL,	NULL,
	"Log directory for DEFEDB"
	SM_LC_NO_ISSET	SM_LC_SET_MAGIC(0) },

{ SM_CONF_DEF_MAGIC, QMEDB_CNF_PAGESIZE,	sm_conf_type_u32,
	offsetof(qmgr_cnf_T, q_cnf_edb_cnf.edbcnf_pagesize),
	sizeof(uint),
	NULL,	0,	sm_conf_byte_suffixes,	NULL,	NULL,
	"DB page size"
	SM_LC_NO_ISSET	SM_LC_SET_MAGIC(0) },

{ SM_CONF_DEF_MAGIC, QMEDB_CNF_CACHESIZE,	sm_conf_type_u32,
	offsetof(qmgr_cnf_T, q_cnf_edb_cnf.edbcnf_cachesize),
	sizeof(uint),
	SM_XSTR(EDB_CACHE_DFLT), 0,	sm_conf_byte_suffixes,	NULL,	NULL,
	"DB cache size"
	SM_LC_NO_ISSET	SM_LC_SET_MAGIC(0) },

{ SM_CONF_DEF_MAGIC, QMEDB_CNF_CHKP_KB,	sm_conf_type_u32,
	offsetof(qmgr_cnf_T, q_cnf_edb_cnf.edbcnf_chkpt_kb),
	sizeof(uint32_t),
	NULL,	0,	sm_conf_Kbyte_suffixes,	NULL,	NULL,
	"If non-zero, a checkpoint will be done if more than the amount of"
	" KBytes of log data have been written since the last checkpoint"
	SM_LC_NO_ISSET	SM_LC_SET_MAGIC(0) },

{ SM_CONF_DEF_MAGIC, QMEDB_CNF_CHKP_DELAY,	sm_conf_type_u32,
	offsetof(qmgr_cnf_T, q_cnf_edb_cnf.edbcnf_chkpt_delay),
	sizeof(uint32_t),
	SM_XSTR(EDB_SWEEP_DELAY), 0,	sm_conf_time_suffixes,	NULL,	NULL,
	"Minimum delay between two checkpoints"
	SM_LC_NO_ISSET	SM_LC_SET_MAGIC(0) },

{ SM_CONF_DEF_MAGIC, QMGR_CNF_FLAGS,			sm_conf_type_choice,
	offsetof(qmgr_cnf_T, q_cnf_edb_cnf.edbcnf_oflags),	sizeof(uint),
	NULL,	SM_CONF_FLAG_MULTIPLE|SM_CONF_FLAG_OR|SM_CONF_FLAG_PLURAL,
	qmgr_edb_flag_names,	NULL,	NULL,
	"flags for DEFEDB"
	SM_LC_NO_ISSET	SM_LC_SET_MAGIC(0) },

{ SM_CONF_DEF_MAGIC, QMGR_CNF_EDBC_HASH_TABLE_SIZE,		sm_conf_type_u32,
	offsetof(qmgr_cnf_T, q_cnf_edbc_ht_size),	sizeof(uint),
	SM_XSTR(EDBC_HT_INIT),	0,	edbc_ht_constraints,	NULL,	NULL,
	"initial size for in-memory cache"
	SM_LC_NO_ISSET
	SM_LC_SET_MAGIC(SM_QMGR_CNF_MAGIC)
	},

CONF_SENTINEL
}
#endif /* SM_QMGRCNFDEF */
;

EXTERN
sm_conf_definition_T
qmgr_ibdb_defs[]
#if SM_QMGRCNFDEF
=
{
{ SM_CONF_DEF_MAGIC, QMGR_CNF_IBDB_SIZE,	sm_conf_type_u32,
	offsetof(qmgr_cnf_T, q_cnf_ibdb_size),	sizeof(uint),
	SM_XSTR(IBDB_SIZE),	0,	sm_conf_byte_suffixes,
	NULL,	NULL,
	"maximum size of each IBDB file"
	SM_LC_NO_ISSET	SM_LC_SET_MAGIC(0) },
{ SM_CONF_DEF_MAGIC, QMGR_CNF_OPEN_TAS_MAX,		sm_conf_type_u32,
	offsetof(qmgr_cnf_T, q_cnf_optas),		sizeof(uint),
	SM_XSTR(IBDB_OPEN_TAS),	0,	NULL,	NULL,	NULL,
	"maximum number of open transactions in IBDB before a commit is performed"
	SM_LC_NO_ISSET	SM_LC_SET_MAGIC(0) },
{ SM_CONF_DEF_MAGIC, QMGR_CNF_IBDB_COMMIT_DELAY_MAX,	sm_conf_type_u32,
	offsetof(qmgr_cnf_T, q_cnf_ibdb_delay),	sizeof(uint),
	SM_XSTR(IBDB_DELAY),	0,	NULL,	NULL,	NULL,
	"maximum time between commits to IBDB (micro seconds)"
	SM_LC_NO_ISSET	SM_LC_SET_MAGIC(0) },

CONF_SENTINEL
}
#endif /* SM_QMGRCNFDEF */
;


EXTERN
sm_conf_definition_T const
qmgr_sc_flag_names[]
#if SM_QMGRCNFDEF
=
{
#if MTA_USE_TLS
{ SM_CONF_DEF_MAGIC,
	QMGR_CNF_SC_LKP_RCPT_CONF,			sm_conf_type_choice_value,
	QCNF_FL_SC_LKP_RCPT_CONF,
	0, NULL, 0, NULL, NULL, NULL,
	"lookup recipient configuration data in access map"
	SM_LC_NO_ISSET	SM_LC_SET_MAGIC(0) },

{ SM_CONF_DEF_MAGIC,
	QMGR_CNF_SC_LKP_SE_CONF,			sm_conf_type_choice_value,
	QCNF_FL_SC_LKP_SE_CONF,
	0, NULL, 0, NULL, NULL, NULL,
	"lookup session configuration data in qmgr_conf map"
	SM_LC_NO_ISSET	SM_LC_SET_MAGIC(0) },
#endif

CONF_SENTINEL
}
#endif /* SM_QMGRCNFDEF */
;


EXTERN
sm_conf_definition_T const
qmgr_smtpc_lfl_names[]
#if SM_QMGRCNFDEF
=
{
	MAP_FL_CNF_DEFS,

CONF_SENTINEL
}
#endif /* SM_QMGRCNFDEF */
;


EXTERN
sm_conf_definition_T
qmgr_smtpc_defs[]
#if SM_QMGRCNFDEF
=
{
{ SM_CONF_DEF_MAGIC, QMGR_CNF_SC_INITIAL_CONNS,	sm_conf_type_u32,
	offsetof(qmgr_cnf_T, q_cnf_init_conc_conn),	sizeof(uint),
	SM_XSTR(DA_INIT_CONC_CONN),	0,	NULL,	NULL,	NULL,
	"initial number of outgoing connections to a single host"
	SM_LC_NO_ISSET	SM_LC_SET_MAGIC(0) },
{ SM_CONF_DEF_MAGIC, QMGR_CNF_SC_CONNS_MAX,	sm_conf_type_u32,
	offsetof(qmgr_cnf_T, q_cnf_max_conc_conn),	sizeof(uint),
	SM_XSTR(DA_CONC_CONN_MAX),	0,	NULL,	NULL,	NULL,
	"maximum number of outgoing connections to a single host"
	SM_LC_NO_ISSET	SM_LC_SET_MAGIC(0) },

{ SM_CONF_DEF_MAGIC, QMGR_CNF_SC_TAS_SE,	sm_conf_type_u32,
	offsetof(qmgr_cnf_T, q_cnf_tas_se),	sizeof(uint),
	SM_XSTR(DA_TAS_SE_MAX),	0,	NULL,	NULL,	NULL,
	"maximum number of transactions per session"
	SM_LC_NO_ISSET	SM_LC_SET_MAGIC(0) },

{ SM_CONF_DEF_MAGIC, QMGR_CNF_SC_SMTP_RCPTS_TA,	sm_conf_type_u32,
	offsetof(qmgr_cnf_T, q_cnf_smtp_rcpts_ta),	sizeof(uint),
	SM_XSTR(DA_RCPTS_TA_MAX),	0,	NULL,	NULL,	NULL,
	"maximum number of recipients per transaction (smtp)"
	SM_LC_NO_ISSET	SM_LC_SET_MAGIC(0) },
{ SM_CONF_DEF_MAGIC, QMGR_CNF_SC_LMTP_RCPTS_TA,	sm_conf_type_u32,
	offsetof(qmgr_cnf_T, q_cnf_lmtp_rcpts_ta),	sizeof(uint),
	SM_XSTR(DA_RCPTS_TA_MAX),	0,	NULL,	NULL,	NULL,
	"maximum number of recipients per transaction (lmtp)"
	SM_LC_NO_ISSET	SM_LC_SET_MAGIC(0) },

#if 0
{ SM_CONF_DEF_MAGIC, "message_rate_max",		sm_conf_type_u32,
	offsetof(qmgr_cnf_T, sc_cnf_max_conn_rate),	sizeof(uint32_t),
	"100",	0,
	NULL, NULL,	NULL,
	"global max rate"
	SM_LC_NO_ISSET	SM_LC_SET_MAGIC(0) },
#endif

{ SM_CONF_DEF_MAGIC, "connection_limits",		sm_conf_type_array,
	0,	/* offset */
	0,	/* size (# of elements) */
	NULL,	/* default */
	SM_CONF_FLAG_MULTIPLE|SM_CONF_FLAG_DEFAULT_FROM_ENVIRONMENT|SM_CONF_FLAG_SECTION_DEFAULT_FROM_ANONYMOUS,
	sc_cl_definitions,
	NULL,  NULL,
	"connection limit declarations"
	SM_LC_NO_ISSET	SM_LC_SET_MAGIC(0) },

{ SM_CONF_DEF_MAGIC, QMGR_CNF_FLAGS,			sm_conf_type_choice,
	offsetof(qmgr_cnf_T, q_cnf_flags),		sizeof(uint32_t),
	NULL,	SM_CONF_FLAG_MULTIPLE|SM_CONF_FLAG_OR|SM_CONF_FLAG_PLURAL,
	qmgr_sc_flag_names,	NULL,	NULL,
	""
	SM_LC_NO_ISSET
	SM_LC_SET_MAGIC(SM_QMGR_CNF_MAGIC)
	},

{ SM_CONF_DEF_MAGIC, "rcpt_conf_lookup_flags",			sm_conf_type_choice,
	offsetof(qmgr_cnf_T, q_cnf_sc_lfl),		sizeof(uint32_t),
	NULL,	SM_CONF_FLAG_MULTIPLE|SM_CONF_FLAG_OR|SM_CONF_FLAG_PLURAL,
	qmgr_smtpc_lfl_names,	NULL,	NULL,
	""
	SM_LC_NO_ISSET
	SM_LC_SET_MAGIC(SM_QMGR_CNF_MAGIC)
	},



CONF_SENTINEL
}
#endif /* SM_QMGRCNFDEF */
;

EXTERN
sm_conf_definition_T
qmgr_smtps_defs[]
#if SM_QMGRCNFDEF
=
{
{ SM_CONF_DEF_MAGIC, QMGR_CNF_SS_OPEN_CONNS_MAX,	sm_conf_type_u32,
	offsetof(qmgr_cnf_T, q_cnf_max_open_se),	sizeof(uint),
	SM_XSTR(SS_OPEN_SE_MAX),	0,	NULL,	NULL,	NULL,
	"maximum number of open incoming connection from a single host"
	SM_LC_NO_ISSET	SM_LC_SET_MAGIC(0) },
{ SM_CONF_DEF_MAGIC, QMGR_CNF_SS_CONN_RATE_MAX,	sm_conf_type_u32,
	offsetof(qmgr_cnf_T, q_cnf_max_conn_rate),	sizeof(uint),
	SM_XSTR(SS_CONN_RATE_MAX),	0,	NULL,	NULL,	NULL,
	"maximum incoming connection rate from a single host"
	SM_LC_NO_ISSET	SM_LC_SET_MAGIC(0) },
{ SM_CONF_DEF_MAGIC, QMGR_CNF_SS_CONNCTL_SIZE,	sm_conf_type_u32,
	offsetof(qmgr_cnf_T, q_cnf_ss_cc_size),	sizeof(uint),
	SM_XSTR(SS_CONNCTL_SIZE),	0,	NULL,	NULL,	NULL,
	"size of connection control hash table [experts only]"
	SM_LC_NO_ISSET	SM_LC_SET_MAGIC(0) },

CONF_SENTINEL
}
#endif /* SM_QMGRCNFDEF */
;


EXTERN
sm_conf_definition_T
qmgr_iqdb_defs[]
#if SM_QMGRCNFDEF
=
{
{ SM_CONF_DEF_MAGIC, QMGR_CNF_IQDB_CACHE_SIZE,	sm_conf_type_u32,
	offsetof(qmgr_cnf_T, q_cnf_iqdb_csize), sizeof(uint),
	SM_XSTR(IQDB_CSIZE),	0,	NULL,	NULL,	NULL,
	"maximum number of entries in IQDB cache"
	SM_LC_NO_ISSET	SM_LC_SET_MAGIC(0) },
{ SM_CONF_DEF_MAGIC, QMGR_CNF_IQDB_HASH_TABLE_SIZE,	sm_conf_type_u32,
	offsetof(qmgr_cnf_T, q_cnf_iqdb_htsize),	sizeof(uint),
	SM_XSTR(IQDB_HTSIZE),	0,	NULL,	NULL,	NULL,
	"size of hash table for IQDB"
	SM_LC_NO_ISSET	SM_LC_SET_MAGIC(0) },

CONF_SENTINEL
}
#endif /* SM_QMGRCNFDEF */
;

EXTERN
sm_conf_definition_T const
qmgr_verp_when_names[]
#if SM_QMGRCNFDEF
=
{
/*
 *  must be a choice of some values:
 *  never, xverp, owner, always
 *  but xverp and owner can be set together...
 *  simplify for now: just three choices:
 *  never, when_requested, always
 */

{ SM_CONF_DEF_MAGIC, "never",	sm_conf_type_choice_value,
	QCNF_FL_VERP_NEVER,
	0, NULL, 0, NULL, NULL, NULL,
	"never use VERP"
	SM_LC_NO_ISSET	SM_LC_SET_MAGIC(0) },

{ SM_CONF_DEF_MAGIC, "when_requested",	sm_conf_type_choice_value,
	0,
	0, NULL, 0, NULL, NULL, NULL,
	"use VERP when requested via XVERP or verp- in aliases"
	SM_LC_NO_ISSET	SM_LC_SET_MAGIC(0) },

{ SM_CONF_DEF_MAGIC, "always",	sm_conf_type_choice_value,
	QCNF_FL_VERP_ALWAYS,
	0, NULL, 0, NULL, NULL, NULL,
	"always use VERP"
	SM_LC_NO_ISSET	SM_LC_SET_MAGIC(0) },

CONF_SENTINEL
}
#endif /* SM_QMGRCNFDEF */
;

EXTERN
sm_conf_definition_T const
qmgr_verp_add_names[]
#if SM_QMGRCNFDEF
=
{
{ SM_CONF_DEF_MAGIC, "rcpt",	sm_conf_type_choice_value,
	QCNF_FL_VERP_RCPT,
	0, NULL, 0, NULL, NULL, NULL,
	"add (encoded) RCPT address"
	SM_LC_NO_ISSET	SM_LC_SET_MAGIC(0) },

CONF_SENTINEL
}
#endif /* SM_QMGRCNFDEF */
;

EXTERN
sm_conf_definition_T
qmgr_verp_defs[]
#if SM_QMGRCNFDEF
=
{
{ SM_CONF_DEF_MAGIC, "when",		sm_conf_type_choice,
	offsetof(qmgr_cnf_T, q_cnf_flags),		sizeof(uint32_t),
	NULL,	SM_CONF_FLAG_OR,
	qmgr_verp_when_names,	NULL,	NULL,
	"when to use VERP"
	SM_LC_NO_ISSET	SM_LC_SET_MAGIC(0) },

{ SM_CONF_DEF_MAGIC, "add",		sm_conf_type_choice,
	offsetof(qmgr_cnf_T, q_cnf_flags),		sizeof(uint32_t),
	"rcpt",	SM_CONF_FLAG_MULTIPLE|SM_CONF_FLAG_OR,
	qmgr_verp_add_names,	NULL,	NULL,
	"what to add to sender address"
	SM_LC_NO_ISSET	SM_LC_SET_MAGIC(0) },

{ SM_CONF_DEF_MAGIC, "delimiter1",		sm_conf_type_char,
	offsetof(qmgr_cnf_T, q_cnf_verp1),		sizeof(uchar),
	NULL,	0,	NULL,	NULL,	NULL,
	"first delimiter for VERP extension in local part"
	SM_LC_NO_ISSET	SM_LC_SET_MAGIC(0) },

{ SM_CONF_DEF_MAGIC, "delimiter2",		sm_conf_type_char,
	offsetof(qmgr_cnf_T, q_cnf_verp2),		sizeof(uchar),
	NULL,	0,	NULL,	NULL,	NULL,
	"second delimiter for VERP extension in local part"
	SM_LC_NO_ISSET	SM_LC_SET_MAGIC(0) },

{ SM_CONF_DEF_MAGIC, "delimiter3",		sm_conf_type_char,
	offsetof(qmgr_cnf_T, q_cnf_verp3),		sizeof(uchar),
	NULL,	0,	NULL,	NULL,	NULL,
	"third delimiter for VERP extension in local part"
	SM_LC_NO_ISSET	SM_LC_SET_MAGIC(0) },

CONF_SENTINEL
}
#endif /* SM_QMGRCNFDEF */
;


EXTERN
sm_conf_definition_T
qmgr_defs[]
#if SM_QMGRCNFDEF
=
{
{ SM_CONF_DEF_MAGIC, QMGR_CNF_AQ_SIZE,		sm_conf_type_u32,
	offsetof(qmgr_cnf_T, q_cnf_aq_size),	sizeof(uint),
	SM_XSTR(AQ_SIZE),	0,	NULL,	NULL,	NULL,
	"maximum number of entries in AQ (active queue)"
	SM_LC_NO_ISSET
	SM_LC_SET_MAGIC(SM_QMGR_CNF_MAGIC)
	},

{ SM_CONF_DEF_MAGIC, QMGR_CNF_OCC_SIZE,		sm_conf_type_u32,
	offsetof(qmgr_cnf_T, q_cnf_occ_size),	sizeof(uint),
	SM_XSTR(OCC_CSIZE),	0,	NULL,	NULL,	NULL,
	"maximum number of entries in OCC (outgoing connection cache)"
	SM_LC_NO_ISSET
	SM_LC_SET_MAGIC(SM_QMGR_CNF_MAGIC)
	},

{ SM_CONF_DEF_MAGIC, "IBDB", sm_conf_type_section,
	0, sizeof(qmgr_cnf_T), NULL, SM_CONF_FLAG_KEEP_DEFAULT,
	qmgr_ibdb_defs, NULL, NULL, NULL
	SM_LC_NO_ISSET	SM_LC_SET_MAGIC(0) },

{ SM_CONF_DEF_MAGIC, "IQDB", sm_conf_type_section,
	0, sizeof(qmgr_cnf_T), NULL, SM_CONF_FLAG_KEEP_DEFAULT,
	qmgr_iqdb_defs, NULL, NULL, NULL
	SM_LC_NO_ISSET	SM_LC_SET_MAGIC(0) },

{ SM_CONF_DEF_MAGIC, QMGR_CNF_DISK_SPACE_MIN,		sm_conf_type_u32,
	offsetof(qmgr_cnf_T, q_cnf_min_df),		sizeof(uint),
	SM_XSTR(DISK_FREE_MIN),	0,	sm_conf_Kbyte_suffixes,
	NULL,	NULL,
	"minimum amount of free disk space (KB)"
	SM_LC_NO_ISSET
	SM_LC_SET_MAGIC(SM_QMGR_CNF_MAGIC)
	},

{ SM_CONF_DEF_MAGIC, QMGR_CNF_OK_DISK_SPACE,		sm_conf_type_u32,
	offsetof(qmgr_cnf_T, q_cnf_ok_df),		sizeof(uint),
	SM_XSTR(DISK_OK_FREE),	0,	sm_conf_Kbyte_suffixes,
	NULL,	NULL,
	"amount of free disk space at which normal operation continues (KB)"
	SM_LC_NO_ISSET
	SM_LC_SET_MAGIC(SM_QMGR_CNF_MAGIC)
	},

{ SM_CONF_DEF_MAGIC, "smtpc", sm_conf_type_section,
	0, sizeof(qmgr_cnf_T), NULL, SM_CONF_FLAG_KEEP_DEFAULT,
	qmgr_smtpc_defs, NULL, NULL, NULL
	SM_LC_NO_ISSET	SM_LC_SET_MAGIC(0) },

{ SM_CONF_DEF_MAGIC, "smtps", sm_conf_type_section,
	0, sizeof(qmgr_cnf_T), NULL, SM_CONF_FLAG_KEEP_DEFAULT,
	qmgr_smtps_defs, NULL, NULL, NULL
	SM_LC_NO_ISSET	SM_LC_SET_MAGIC(0) },

{ SM_CONF_DEF_MAGIC, QMGR_CNF_FLAGS,			sm_conf_type_choice,
	offsetof(qmgr_cnf_T, q_cnf_flags),		sizeof(uint32_t),
	NULL,	SM_CONF_FLAG_MULTIPLE|SM_CONF_FLAG_OR|SM_CONF_FLAG_PLURAL,
	qmgr_flag_names,	NULL,	NULL,
	""
	SM_LC_NO_ISSET
	SM_LC_SET_MAGIC(SM_QMGR_CNF_MAGIC)
	},

{ SM_CONF_DEF_MAGIC, QMGR_CNF_DELAY_RETRIES_MIN,	sm_conf_type_u32,
	offsetof(qmgr_cnf_T, q_cnf_min_delay),	sizeof(uint),
	SM_XSTR(DELAY_NXT_TRY_MIN),	0,	sm_conf_time_suffixes,
	NULL,	NULL,
	"minimum time for retrying a delivery"
	SM_LC_NO_ISSET
	SM_LC_SET_MAGIC(SM_QMGR_CNF_MAGIC)
	},
{ SM_CONF_DEF_MAGIC, QMGR_CNF_DELAY_RETRIES_MAX,	sm_conf_type_u32,
	offsetof(qmgr_cnf_T, q_cnf_max_delay),	sizeof(uint),
	SM_XSTR(DELAY_NXT_TRY_MAX),	0,	sm_conf_time_suffixes,
	NULL,	NULL,
	"maximum time for retrying a delivery"
	SM_LC_NO_ISSET
	SM_LC_SET_MAGIC(SM_QMGR_CNF_MAGIC)
	},

{ SM_CONF_DEF_MAGIC, QMGR_CNF_TMOUT_QUEUERETURN,	sm_conf_type_u32,
	offsetof(qmgr_cnf_T, q_cnf_tmo_return),	sizeof(uint),
	SM_XSTR(QM_TMO_RETURN),	0,	sm_conf_time_suffixes,
	NULL,	NULL,
	"maximum time in queue"
	SM_LC_NO_ISSET
	SM_LC_SET_MAGIC(SM_QMGR_CNF_MAGIC)
	},

{ SM_CONF_DEF_MAGIC, QMGR_CNF_TMOUT_QUEUEDELAY,	sm_conf_type_u32,
	offsetof(qmgr_cnf_T, q_cnf_tmo_delay),	sizeof(uint),
	SM_XSTR(QM_TMO_DELAY),	0,	sm_conf_time_suffixes,
	NULL,	NULL,
	"send delay warning after this time in queue"
	SM_LC_NO_ISSET
	SM_LC_SET_MAGIC(SM_QMGR_CNF_MAGIC)
	},

{ SM_CONF_DEF_MAGIC, QMGR_CNF_TMOUT_AR,		sm_conf_type_u32,
	offsetof(qmgr_cnf_T, q_cnf_tmo_ar),	sizeof(uint),
	SM_XSTR(AQR_AR_TMOUT),	0,	sm_conf_time_suffixes,
	NULL,	NULL,
	"timeout in address resolver"
	SM_LC_NO_ISSET
	SM_LC_SET_MAGIC(SM_QMGR_CNF_MAGIC)
	},
{ SM_CONF_DEF_MAGIC, QMGR_CNF_TMOUT_DELIVERY,		sm_conf_type_u32,
	offsetof(qmgr_cnf_T, q_cnf_tmo_da),	sizeof(uint),
	SM_XSTR(AQR_DA_TMOUT),	0,	sm_conf_time_suffixes,
	NULL,	NULL,
	"timeout for a single delivery attempt"
	SM_LC_NO_ISSET
	SM_LC_SET_MAGIC(SM_QMGR_CNF_MAGIC)
	},
{ SM_CONF_DEF_MAGIC, QMGR_CNF_TMOUT_SCHED,		sm_conf_type_u32,
	offsetof(qmgr_cnf_T, q_cnf_tmo_sched),	sizeof(uint),
	SM_XSTR(AQR_SCHED_TMOUT),	0,	sm_conf_time_suffixes,
	NULL,	NULL,
	"maximum time waiting to be scheduled"
	SM_LC_NO_ISSET
	SM_LC_SET_MAGIC(SM_QMGR_CNF_MAGIC)
	},

{ SM_CONF_DEF_MAGIC, QMGR_CNF_WAIT_FOR_SERVER,		sm_conf_type_u32,
	offsetof(qmgr_cnf_T, q_cnf_wait4srv),	sizeof(uint),
	"1",	0,	sm_conf_time_suffixes,
	NULL,	NULL,
	"maximum amount of time to wait for a server to become available"
	SM_LC_NO_ISSET
	SM_LC_SET_MAGIC(SM_QMGR_CNF_MAGIC)
	},
{ SM_CONF_DEF_MAGIC, QMGR_CNF_WAIT_FOR_CLIENT,		sm_conf_type_u32,
	offsetof(qmgr_cnf_T, q_cnf_wait4clt),	sizeof(uint),
	"1",	0,	sm_conf_time_suffixes,
	NULL,	NULL,
	"maximum amount of time to wait for a client to become available"
	SM_LC_NO_ISSET
	SM_LC_SET_MAGIC(SM_QMGR_CNF_MAGIC)
	},

{ SM_CONF_DEF_MAGIC, QMGR_CNF_LOG_LEVEL,		sm_conf_type_u32,
	offsetof(qmgr_cnf_T, q_cnf_loglevel),	sizeof(uint),
	SM_XSTR(SM_LOG_LEVEL),	0 /* SM_CONF_FLAG_DPRCD */,	NULL,	NULL,	NULL,
	""
	SM_LC_NO_ISSET
	SM_LC_SET_MAGIC(SM_QMGR_CNF_MAGIC)
	},

{ SM_CONF_DEF_MAGIC, "log", sm_conf_type_section,
	offsetof(qmgr_cnf_T, q_cnf_log),	0,
	NULL, SM_CONF_FLAG_KEEP_DEFAULT, sm_log_spec_defs,
	NULL,	NULL,
	SM_CNF_LOG
	SM_LC_NO_ISSET
	SM_LC_SET_MAGIC(SM_QMGR_CNF_MAGIC)
	},

#if QMGR_DEBUG
{ SM_CONF_DEF_MAGIC, QMGR_CNF_DEBUG_LEVEL,		sm_conf_type_u32,
	offsetof(qmgr_cnf_T, q_cnf_debug),	sizeof(uint),
	"0", 0, NULL, NULL, NULL, NULL
	SM_LC_NO_ISSET
	SM_LC_SET_MAGIC(SM_QMGR_CNF_MAGIC)
	},
#endif /* QMGR_DEBUG */

#if QMGR_TEST
{ SM_CONF_DEF_MAGIC, QMGR_CNF_TESTS,		sm_conf_type_u32,
	offsetof(qmgr_cnf_T, q_cnf_tests),	sizeof(uint),
	"0", 0, NULL, NULL, NULL, NULL
	SM_LC_NO_ISSET	SM_LC_SET_MAGIC(0) },

{ SM_CONF_DEF_MAGIC, "test_total_transactions",		sm_conf_type_u32,
	offsetof(qmgr_cnf_T, q_cnf_tot_tas),	sizeof(uint32_t),
	"5000", 0, NULL, NULL, NULL, NULL
	SM_LC_NO_ISSET	SM_LC_SET_MAGIC(0) },

{ SM_CONF_DEF_MAGIC, "test_once_transactions",		sm_conf_type_u32,
	offsetof(qmgr_cnf_T, q_cnf_once_tas),	sizeof(uint32_t),
	"100", 0, NULL, NULL, NULL, NULL
	SM_LC_NO_ISSET	SM_LC_SET_MAGIC(0) },

{ SM_CONF_DEF_MAGIC, "test_fill_AQ_max",		sm_conf_type_u32,
	offsetof(qmgr_cnf_T, q_cnf_max_fill_aq),	sizeof(uint),
	"80", 0, NULL, NULL, NULL, NULL
	SM_LC_NO_ISSET	SM_LC_SET_MAGIC(0) },

{ SM_CONF_DEF_MAGIC, "test_fill_rate",		sm_conf_type_u32,
	offsetof(qmgr_cnf_T, q_cnf_fill_rate),	sizeof(uint),
	"1000", 0, NULL, NULL, NULL, NULL
	SM_LC_NO_ISSET	SM_LC_SET_MAGIC(0) },

#endif /* QMGR_TEST */

{ SM_CONF_DEF_MAGIC, "DSN_handling",	sm_conf_type_section,
	0, sizeof(qmgr_cnf_T), NULL, SM_CONF_FLAG_KEEP_DEFAULT, qmgr_dsn_defs,
	NULL, NULL, NULL
	SM_LC_NO_ISSET	SM_LC_SET_MAGIC(0) },

{ SM_CONF_DEF_MAGIC, "VERP",	sm_conf_type_section,
	0, sizeof(qmgr_cnf_T), NULL, SM_CONF_FLAG_KEEP_DEFAULT, qmgr_verp_defs,
	NULL, NULL, NULL
	SM_LC_NO_ISSET	SM_LC_SET_MAGIC(0) },

{ SM_CONF_DEF_MAGIC, "DEFEDB", sm_conf_type_section,
	0, sizeof(edb_cnf_T), NULL, SM_CONF_FLAG_KEEP_DEFAULT, qmgr_edb_defs,
	NULL, NULL, NULL
	SM_LC_NO_ISSET	SM_LC_SET_MAGIC(0) },

{ SM_CONF_DEF_MAGIC, "control_socket",	sm_conf_type_string,
	offsetof(qmgr_cnf_T, q_cnf_ctlsock),	0,
	NULL, 0, NULL, NULL, NULL,
	"name of control socket"
	SM_LC_NO_ISSET
	SM_LC_SET_MAGIC(SM_QMGR_CNF_MAGIC)
	},

{ SM_CONF_DEF_MAGIC, "conf",	sm_conf_type_string,
	offsetof(qmgr_cnf_T, q_cnf_confmap_fn),	0,
	"qmgr_conf.db", 0, NULL, NULL, NULL,
	"name of configuration map (including extension)"
	SM_LC_NO_ISSET
	SM_LC_SET_MAGIC(SM_QMGR_CNF_MAGIC)
	},

{ SM_CONF_DEF_MAGIC, "double_bounce_address",	sm_conf_type_string,
	offsetof(qmgr_cnf_T, q_cnf_pm_add),	0,
	NULL, 0, NULL, NULL, NULL,
	"full address for double bounces (defaults to <doublebounce@HOSTNAME>)"
	SM_LC_NO_ISSET
	SM_LC_SET_MAGIC(SM_QMGR_CNF_MAGIC)
	},

{ SM_CONF_DEF_MAGIC, EVTHR_CNF_THR_MIN,		sm_conf_type_u32,
	offsetof(qmgr_cnf_T, q_cnf_min_thr),	sizeof(uint),
	SM_XSTR(SM_QMGR_THR_MIN),	0,	NULL,	NULL,	NULL,
	EVTHR_COMM_THR_MIN
	SM_LC_NO_ISSET
	SM_LC_SET_MAGIC(SM_QMGR_CNF_MAGIC)
	},
{ SM_CONF_DEF_MAGIC, EVTHR_CNF_THR_MAX,		sm_conf_type_u32,
	offsetof(qmgr_cnf_T, q_cnf_max_thr),	sizeof(uint),
	SM_XSTR(SM_QMGR_THR_MAX),	0,	NULL,	NULL,	NULL,
	EVTHR_COMM_THR_MAX
	SM_LC_NO_ISSET
	SM_LC_SET_MAGIC(SM_QMGR_CNF_MAGIC)
	},
{ SM_CONF_DEF_MAGIC, EVTHR_CNF_FD_MAX,		sm_conf_type_u32,
	offsetof(qmgr_cnf_T, q_cnf_max_fd),	sizeof(uint),
	SM_XSTR(SM_QMGR_FD_MAX),	0,	NULL,	NULL,	NULL,
	EVTHR_COMM_FD_MAX
	SM_LC_NO_ISSET
	SM_LC_SET_MAGIC(SM_QMGR_CNF_MAGIC)
	},

	MCP_DEFS(qmgr_cnf_T, q_cnf)

CONF_SENTINEL
}
#endif /* SM_QMGRCNFDEF */
;

EXTERN
sm_conf_definition_T
qmgr_global_defs[]
#if SM_QMGRCNFDEF
=
{
{ SM_CONF_DEF_MAGIC, "SMTPC_socket",	sm_conf_type_string,
	offsetof(qmgr_cnf_T, q_cnf_smtpcsock),	0,	smsmtpcsock,
	0, NULL, NULL, NULL, NULL
	SM_LC_NO_ISSET
	SM_LC_SET_MAGIC(SM_QMGR_CNF_MAGIC)
	},
{ SM_CONF_DEF_MAGIC, "SMTPS_socket",	sm_conf_type_string,
	offsetof(qmgr_cnf_T, q_cnf_smtpssock),	0,	smsmtpssock,
	0, NULL, NULL, NULL, NULL
	SM_LC_NO_ISSET
	SM_LC_SET_MAGIC(SM_QMGR_CNF_MAGIC)
	},
{ SM_CONF_DEF_MAGIC, "SMAR_socket",	sm_conf_type_string,
	offsetof(qmgr_cnf_T, q_cnf_smarsock),	0,	smarsock,
	0, NULL, NULL, NULL, NULL
	SM_LC_NO_ISSET
	SM_LC_SET_MAGIC(SM_QMGR_CNF_MAGIC)
	},
{ SM_CONF_DEF_MAGIC, "CDB_base_directory",	sm_conf_type_string,
	offsetof(qmgr_cnf_T, q_cnf_cdb_base),	0,	"",
	0, NULL, NULL, NULL, NULL
	SM_LC_NO_ISSET
	SM_LC_SET_MAGIC(SM_QMGR_CNF_MAGIC)
	},
{ SM_CONF_DEF_MAGIC, "hostname",		sm_conf_type_string,
	offsetof(qmgr_cnf_T, q_cnf_hostname),	0,	NULL,
	0, NULL, NULL, NULL, NULL
	SM_LC_NO_ISSET
	SM_LC_SET_MAGIC(SM_QMGR_CNF_MAGIC)
	},

{ SM_CONF_DEF_MAGIC, "qmgr", sm_conf_type_section,
	0, sizeof(qmgr_cnf_T), NULL, SM_CONF_FLAG_KEEP_DEFAULT, qmgr_defs,
	NULL, NULL, NULL
	SM_LC_NO_ISSET	SM_LC_SET_MAGIC(0) },

CONF_SENTINEL
}
#endif /* SM_QMGRCNFDEF */
;

#undef EXTERN

#endif /* SM_QMGRCNFDEF_H */
