/*
 * Copyright (c) 2005, 2006, 2011 Sendmail, Inc. and its suppliers.
 *	All rights reserved.
 *
 * By using this file, you agree to the terms and conditions set
 * forth in the LICENSE file which can be found at the top level of
 * the sendmail distribution.
 *
 * $Id: tlscnf.h,v 1.24 2014/03/04 20:27:09 ca Exp $
 */

#ifndef TLSCNF_H
#define TLSCNF_H 1

#include "sm/generic.h"

#if MTA_USE_TLS
#include "sm/tlsstr.h"

enum dhparam_type_E
{
	DHPBITS  = 1,
	DHPFILE  = 2
};

typedef enum dhparam_type_E dhparam_type_T;
typedef union dhparam_U
{
	dhparam_type_T dhparam_type;

	struct {
	       dhparam_type_T   dhpfile_type;
	       char            *dhpfile_name;
	} dhpfile;

	struct {
	       dhparam_type_T   dhbits_type;
	       int              dhbits_bits;
	} dhbits;

} dhparam_T;


/*
 *  TLS configuration context: parts of it are filled in by TLS_GEN_PATHS(),
 *  using sm_gen_conf_path(), after conf file is read.
 *  This struct is passed to sm_tls_init() which uses it to initialize
 *  the OpenSSL context (SSL_CTX).
 */

struct tls_cnf_S
{
	char		*tlscnf_cert;
	char		*tlscnf_key;
	char		*tlscnf_dsa_cert;
	char		*tlscnf_dsa_key;
	char		*tlscnf_ec_cert;
	char		*tlscnf_ec_key;
	char		*tlscnf_ec_curve;
	char		*tlscnf_certpath;
	char		*tlscnf_cacert;
	char		*tlscnf_calist;
	char		*tlscnf_crlfile;
	char		*tlscnf_crlpath;
	char		*tlscnf_cipherlist;		/* smtpc: per session too */
	dhparam_T	 tlscnf_dhparam;
	char		*tlscnf_fp_dig;	/* digest for cert fingerprints */

	char		*tlscnf_cert_abs;
	char		*tlscnf_key_abs;
	char		*tlscnf_dsa_cert_abs;
	char		*tlscnf_dsa_key_abs;
	char		*tlscnf_ec_cert_abs;
	char		*tlscnf_ec_key_abs;
	char		*tlscnf_certpath_abs;
	char		*tlscnf_cacert_abs;
	char		*tlscnf_calist_abs;
	bool		 tlscnf_fips;
	uint32_t	 tlscnf_cflags;		/* smtps only (c: configuration) */
#if MTA_TLS_DEBUG
	uint32_t	 tlscnf_debug;
#endif
	uint		 tlscnf_tlsreqfail;
	uint		 tlscnf_vrfy_depth;

	const char	*tlscnf_certfile;		/* smtpc: per session too */
	const char	*tlscnf_keyfile;		/* smtpc: per session too */
	const char	*tlscnf_dsa_certfile;	/* smtpc: per session too */
	const char	*tlscnf_dsa_keyfile;	/* smtpc: per session too */
	const char	*tlscnf_ec_certfile;
	const char	*tlscnf_ec_keyfile;
	const char	*tlscnf_cacertpath;
	const char	*tlscnf_cacertfile;
	const char	*tlscnf_calistfile;
	uint		 tlscnf_cache_size;
	uint		 tlscnf_cache_tmout;
	long		 tlscnf_options;		/* smtpc: per session too */
	uchar		 tlscnf_options_isset;
	uchar		 tlscnf_calist_isset;
	uchar		 tlscnf_cert_isset;
	uchar		 tlscnf_key_isset;
	uchar		 tlscnf_dsa_cert_isset;
	uchar		 tlscnf_dsa_key_isset;
	uchar		 tlscnf_vrfy_depth_isset;
	uchar		 tlscnf_fp_dig_isset;
#if TLSCNF_FLAGS_ISSET
	uint32_t		 tlscnf_flags_isset;
#endif
};

#endif /* MTA_USE_TLS */
#endif /* TLSCNF_H */
