#!/bin/sh
# $Id: t-dumpdef-0.sh,v 1.186 2014/03/04 20:27:09 ca Exp $
# Copyright (c) 2004-2006, 2011 Sendmail, Inc. and its suppliers.
#	All rights reserved.
#
# By using this file, you agree to the terms and conditions set
# forth in the LICENSE file which can be found at the top level of
# the sendmail distribution.
#
# ----------------------------------------
# show ("dump") configuration
# uses dumpdef
# ----------------------------------------

test -s stop && exit 1

# source directory of this (and other) shell script
SD=`dirname $0`
CONF=../include/smconf.h

ERRS=0

CNF=smx.conf
OUT=smx.out
OK=smx.ok

PRG=./dumpdef

cat > ${CNF} <<EOF
EOF

# get list of compile time options so we can just check the file later on
${PRG} -VVV > ${OUT}

chkcto()
{
  grep "$@" ${OUT} >/dev/null
}

if chkcto 'MTA_NETINET6'
then
  RF='^(127.0.0.1|0:0:0:0:0:0:0:1)$'
else
  RF='127.0.0.1'
fi

if chkcto 'COMMERCIAL'
then
SSFLAGS="flags=0x00800000u|0x01000000u|0x04000000u;"
IQDB_CSIZE=8192
IQDB_HTSIZE=16383
IBDB_SIZE=1048576
OCC_CSIZE=32768
AQ_SIZE=65536
EDBC_HT_INIT=65536
IBDB_DELAY=30000
else
SSFLAGS="flags=0x00800000u|0x01000000u;"
IQDB_CSIZE=1023
IQDB_HTSIZE=2111
IBDB_SIZE=65536
OCC_CSIZE=2111
AQ_SIZE=4096
EDBC_HT_INIT=256
IBDB_DELAY=1000
fi
IBDB_OPEN_TAS=32

if chkcto SC_SESS_FEATS
then
SC_SESS_FEATS='
# session feature declarations
# session_features are not set by default'
else
SC_SESS_FEATS=''
fi

if chkcto COMMERCIAL && chkcto SC_SESS_FEATS && chkcto MTA_USE_TLS
then
SC_BIND_FEATS='
# source declarations
# source is not set by default'
else
SC_BIND_FEATS=''
fi

if chkcto MTA_USE_SASL
then
AUTHOPTS='auth {
# Flags for Cyrus SASL (AUTH)
# flags are not set by default
# List of SASL mechanisms for which relaying is allowed if a client
# successfully authenticated using one of them
trusted_mechanisms="PLAIN LOGIN DIGEST-MD5 CRAM-MD5";
# user realm
# user_realm is not set by default
}'
else
AUTHOPTS=""
fi

if grep 'define MTA_HAVE_ECDSA  *1' $CONF >/dev/null
then
SSECDSA='# File with EC certificate in PEM format
# EC_cert_file is not set by default
# File with private key for EC certificate in PEM format
# EC_key_file is not set by default
# Name of elliptic curve
# EC_curve is not set by default
'
SCECDSA='# File with EC certificate in PEM format
# EC_cert_file is not set by default
# File with private key for EC certificate in PEM format
# EC_key_file is not set by default
'
else
SSECDSA=''
SCECDSA=''
fi

if chkcto 'MTA_TLS_DEBUG'
then
MTA_TLS_DEBUG='
# debug options [experts only]
# debug are not set by default'
else
MTA_TLS_DEBUG=''
fi

if chkcto 'MTA_USE_TLS'
then
SSTLSOPTS="tls {
# File with certificate in PEM format
cert_file=\"smcert.pem\";
# File with private key for certificate in PEM format
key_file=\"smkey.pem\";
# File with DSA certificate in PEM format
# DSA_cert_file is not set by default
# File with private key for DSA certificate in PEM format
# DSA_key_file is not set by default
# File with CA certificate in PEM format
CAcert_file=\"CAcert.pem\";
# Directory with (symbolic links for) CA certificates in PEM format
CAcert_directory=\"certs\";
# File with CA certificates in PEM format (for acceptable client certs)
CAlist_file=\"CAcert.pem\";
# options for SSL_CTX_set_options [expert only]
options=SSL_OP_ALL;
# Cipher List
# CipherList is not set by default
# Digest for cert fingerprints
Digest=\"sha1\";
${SSECDSA}# flags to influence behavior related to STARTTLS [see documentation]
flags=0x00000004;
# size of TLS session cache
# cache_size is not set by default
# timeout for TLS session cache
# cache_timeout is not set by default
# how to treat TLS requirements violation
requirements_violation=abort;
# maximum checked depth of certificate chain for verification
# verify_depth is not set by default
# Certificate Revocation List file
# CRL_file is not set by default
# Certificate Revocation List directory
# CRL_directory is not set by default${MTA_TLS_DEBUG}
}"
else
SSTLSOPTS=""
fi

if chkcto 'MTA_USE_TLS'
then
SCTLSOPTS="tls {
# File with certificate in PEM format
cert_file=\"smcert.pem\";
# File with private key for certificate in PEM format
key_file=\"smkey.pem\";
# File with DSA certificate in PEM format
# DSA_cert_file is not set by default
# File with private key for DSA certificate in PEM format
# DSA_key_file is not set by default
# File with CA certificate in PEM format
CAcert_file=\"CAcert.pem\";
# Directory with (symbolic links for) CA certificates in PEM format
CAcert_directory=\"certs\";
# options for SSL_CTX_set_options [expert only]
options=SSL_OP_ALL;
# Cipher List
# CipherList is not set by default
# Digest for cert fingerprints
Digest=\"sha1\";
${SCECDSA}# size of TLS session cache
# cache_size is not set by default
# timeout for TLS session cache
# cache_timeout is not set by default
# maximum checked depth of certificate chain for verification
# verify_depth is not set by default
# Certificate Revocation List file
# CRL_file is not set by default
# Certificate Revocation List directory
# CRL_directory is not set by default${MTA_TLS_DEBUG}
}"
else
SCTLSOPTS=""
fi

if chkcto 'MTA_USE_PMILTER'
then
PMILTEROPTS='# policy milter
policy_milter {
# maximum amount of time to wait for a reply from a policy milter
timeout=20;
# policy milter flags [see documentation]
# flags are not set by default
}'
else
PMILTEROPTS=""
fi

if chkcto 'CLIENT_BIND'
then
CLIENT_BIND='
# bind() to this local IP address
# clientaddr is not set by default'
else
CLIENT_BIND=""
fi

if chkcto 'QMGR_TEST'
then
QMGRTESTOPTS='
tests=0;
test_total_transactions=5000;
test_once_transactions=100;
test_fill_AQ_max=80;
test_fill_rate=1000;'
else
QMGRTESTOPTS=""
fi

if chkcto 'SC_TEST'
then
SCTESTOPTS='
# test options
tests=0;'
else
SCTESTOPTS=""
fi

if chkcto 'SS_TEST'
then
SSTESTOPTS='
tests=0;'
else
SSTESTOPTS=""
fi

if chkcto 'SS_COND'
then
SSCONDOPTS='
# check declarations
# check is not set by default'
else
SSCONDOPTS=""
fi

if chkcto SMAR_DEBUG
then
DNSPORT='# nameserver port
port=53;
'
else
DNSPORT='
'
fi

# if chkcto 'MTA_USE_DKIM'
# then
# DKIM='
# dkim_signing {
# # signature method
# # signature_method is not set by default
# # canonicalization algorithm for mail body
# # body_canonicalization is not set by default
# # canonicalization algorithm for mail header
# # header_canonicalization is not set by default
# # list of headers to skip during signing
# # skip_header is not set by default
# # list of headers to sign
# # sign_header is not set by default
# # ignore signing errors?
# # ignore_errors is not set by default
# # list of headers to add to signature even if they are not in the message
# # always_sign_headers is not set by default
# # maximum number of headers allowed
# max_headers=256;
# # total header length allowed
# max_header_len=SM_MAX_HDR_SZ;
# # DKIM selector declarations
# # sign is not set by default
# }'
# else
# DKIM=""
# fi

DKIM=""

# dirty hack to remove options=SSL_OP_ALL because it is expanded
# if DKIM is defined (which includes openssl/ssl.h and hence the
# definition of the macro...)
sed -e 's/^  *//' -e '/^$/d' -e '/^options=/d' > ${OK} <<EOF
SMTPC_socket="qmsmtpc/qmsmtpc";
SMTPS_socket="qmsmtps/qmsmtps";
SMAR_socket="qmsmar/qmsmar";
CDB_base_directory="";
smar {
# DNS related options
DNS {
# List of nameservers (IPv4 addresses)
# nameservers is not set by default
# DNS flags
# flags are not set by default
# DNS query timeout
timeout=2;
# number of timeouts after which the name server is considered unresponsive
timeouts_max=8;
# maximum number of DNS query retries
retries=0;
${DNSPORT}}
# List of nameservers (IPv4 addresses)
# nameserver is not set by default
# DNS_Flags are not set by default
DNS_timeout=2;
log_level=99999;
# minimum number of threads [experts only]
threads_min=2;
# maximum number of threads [experts only]
threads_max=16;
# maximum number of file descriptors [experts only]
fds_max=10;
# DNS based blacklists
# dnsbl is not set by default
# Greylisting options
greylisting {
# how long before greylisted can be "confirmed"
grey_wait=60;
# timeout for greylisted entries (didn't "confirm" within that time)
grey_expire=24 *60*60;
# expire whitelisted entries after this time if necessary
white_expire=33 *60*60*24;
# force whitelisted entries to reconfirm after this time
white_timeout=40 *60*60*24;
# name of main database (including .db extension)
main_DB_name="grey_grey_m.db";
# name of secondary database (including .db extension)
secondary_DB_name="grey_grey_s.db";
# try to expire entries when this limit is reached
expire_limit=10000;
# apply netmask to IPv4 address
netmask=0xFFFFFFFF;
}
# Map declaration
# map is not set by default
# delimiter(s) for address extensions in local part
address_delimiter="+";
# aliases definition
aliases {
# filename of aliases map (including extension)
file="aliases.db";
# aliases lookup flags
# flags are not set by default
# name of aliases map
# name is not set by default
}
# mailertable definition
mailertable {
# filename of mailertable file
file="mt";
# name of mailertable map
# name is not set by default
# mailertable lookup flags
# flags are not set by default
}
# access map definition
access_map {
# filename of access map (including extension)
file="access.db";
# name of access map
# name is not set by default
}
# minimum number of processes
processes_min=1;
# maximum number of processes
processes_max=1;
# run process as user
user="";
# run process as group
group="";
# list of services to restart in case of failure
# restart_dependencies is not set by default
# pass id to process in argument list with this option
# pass_id is not set by default
# use id to create name of logfile: Logdir name id . log
use_id_in_logfile_name=0;
# path to program
path="";
# arguments for program
arguments="";
# working directory (chdir(2))
# working_directory is not set by default
# maximum number of failures
# failure_max is not set by default
# how to handle program
start_action="wait";
}
smtpc {
# port to connect to on server
remote_port=25;
LMTP_socket="lmtpsock";
# maximum number of threads
threads_max=0;
# maximum number of waiting threads
wait_threads_max=8;
# minimum number of waiting threads
wait_threads_min=2;
# flags to influence behavior [see documentation]
# flags are not set by default
log_level=99999;
debug_level=0;
# timeout for SMTP I/O operations
io_timeout=120;
# timeout for communication with other modules (QMGR)
module_timeout=30;
# maximum amount of time to wait for a server to become available
wait_for_server=1;
# unique id
id=1;
# connect only to this IP address [testing]
# connect_only_to is not set by default
${CLIENT_BIND}
${SCTLSOPTS}${SCTESTOPTS}${SC_SESS_FEATS}
# minimum number of processes
processes_min=1;
# maximum number of processes
processes_max=1;
# run process as user
user="";
# run process as group
group="";
# list of services to restart in case of failure
# restart_dependencies is not set by default
# pass id to process in argument list with this option
# pass_id is not set by default
# use id to create name of logfile: Logdir name id . log
use_id_in_logfile_name=0;
# path to program
path="";
# arguments for program
arguments="";
# working directory (chdir(2))
# working_directory is not set by default
# maximum number of failures
# failure_max is not set by default
# how to handle program
start_action="wait";
}
smtps {
# path of Unix domain socket to exchange file descriptor [use with MCP]
# pass_fd_socket is not set by default
# address for daemon to listen on [do not use with MCP]
# daemon_address is not set by default
# length of listen(2) queue [do not use with MCP]
listen_queue=256;
# log level
log_level=99999;
# allow relaying from IPv4 addresses that match this regular expression
relay_from="${RF}";
# allow relaying to addresses that match this regular expression
relay_to="(^<postmaster>|@localhost>)$";
# group id (numeric) to use for CDB files
cdb_gid=0;
# (unique) id for server (if multiple SMTP servers are configured)
id=0;
# maximum number of transactions per session
transactions_max=65535;
# maximum number of recipients per session
recipients_per_session_max=65535;
# maximum number of recipients per transaction
recipients_per_transaction_max=65535;
# maximum number of hops (Received: headers)
hops_max=25;
# maximum message size (KB)
message_size_max=64 *1024;
# maximum number of bad SMTP commands per session
bad_commands_per_session_max=3;
# maximum number of SMTP commands with invalid addresses per session
invalid_addresses_per_session_max=200;
# maximum number of nop SMTP commands between transactions
nop_commands_between_transactions_max=5;
# maximum number of bad SMTP commands per transaction
bad_commands_per_transaction_max=3;
# maximum number of nop SMTP commands in a single transaction
nop_commands_in_transaction_max=5;
# maximum number of SMTP commands with invalid addresses per transaction
invalid_addresses_per_transaction_max=100;
# timeout for SMTP I/O operations
io_timeout=300;
# timeout for communication with other modules
module_timeout=20;
# flags to influence behavior [see documentation]
${SSFLAGS}
# maximum number of threads
threads_max=0;
# maximum number of waiting threads
wait_threads_max=8;
# minimum number of waiting threads
wait_threads_min=2;
# number of processes to start
processes=1;
# maximum amount of time to wait for a server to become available
wait_for_server=1;
# maximum amount of time to wait for a reply from smar
wait_for_smar=20;
# debug level
debug_level=0;
${SSTLSOPTS}
${AUTHOPTS}
${PMILTEROPTS}
# session feature declarations
# session_features are not set by default
# Initial 220 greeting text
# greeting is not set by default${SSCONDOPTS}${SSTESTOPTS}${DKIM}
# listen_socket is not set by default
# minimum number of processes
processes_min=1;
# maximum number of processes
processes_max=1;
# run process as user
user="";
# run process as group
group="";
# list of services to restart in case of failure
# restart_dependencies is not set by default
# pass id to process in argument list with this option
# pass_id is not set by default
# use id to create name of logfile: Logdir name id . log
use_id_in_logfile_name=0;
# path to program
path="";
# arguments for program
arguments="";
# working directory (chdir(2))
# working_directory is not set by default
# maximum number of failures
# failure_max is not set by default
# how to handle program
start_action="wait";
}
qmgr {
# maximum number of entries in AQ (active queue)
AQ_entries_max=${AQ_SIZE};
# maximum number of entries in OCC (outgoing connection cache)
OCC_entries_max=${OCC_CSIZE};
IBDB {
# maximum size of each IBDB file
size=${IBDB_SIZE};
# maximum number of open transactions in IBDB before a commit is performed
open_TAs_max=${IBDB_OPEN_TAS};
# maximum time between commits to IBDB (micro seconds)
commit_delay_max=${IBDB_DELAY};
}
IQDB {
# maximum number of entries in IQDB cache
cache_entries_max=${IQDB_CSIZE};
# size of hash table for IQDB
hash_table_entries=${IQDB_HTSIZE};
}
# minimum amount of free disk space (KB)
disk_space_min=64 *1024;
# amount of free disk space at which normal operation continues (KB)
disk_space_ok=1 *1024*1024;
smtpc {
# initial number of outgoing connections to a single host
initial_connections=10;
# maximum number of outgoing connections to a single host
connections_max=100;
# maximum number of transactions per session
transactions_per_session_max=100;
# maximum number of recipients per transaction (smtp)
smtp_rcpts_per_transaction_max=1000;
# maximum number of recipients per transaction (lmtp)
lmtp_rcpts_per_transaction_max=1000;
# connection limit declarations
# connection_limits is not set by default
# flags are not set by default
# rcpt_conf_lookup_flags are not set by default${SC_BIND_FEATS}
}
smtps {
# maximum number of open incoming connection from a single host
connections_max=100;
# maximum incoming connection rate from a single host
connection_rate_max=100;
# size of connection control hash table [experts only]
connection_control_hash_table_size=256;
}
# flags are not set by default
# minimum time for retrying a delivery
retry_delay_min=256;
# maximum time for retrying a delivery
retry_delay_max=3600;
# maximum time in queue
queue_return_timeout=345600;
# send delay warning after this time in queue
queue_delay_timeout=14400;
# timeout in address resolver
SMAR_timeout=120;
# timeout for a single delivery attempt
delivery_timeout=3000;
# maximum time waiting to be scheduled
scheduler_timeout=900;
# maximum amount of time to wait for a server to become available
wait_for_server=1;
# maximum amount of time to wait for a client to become available
wait_for_client=1;
log_level=99999;
debug_level=0;${QMGRTESTOPTS}
DSN_handling {
# maximum number of error messages (failed recipients) in a bounce (DSN)
errors_per_DSN_max=100;
# maximum time to wait for merging multiple DSNs into one
merge_delay_max=1;
# flags for DSN handling
# flags are not set by default
}
VERP {
# when to use VERP
# when is not set by default
# what to add to sender address
add=rcpt;
# first delimiter for VERP extension in local part
# delimiter1 is not set by default
# second delimiter for VERP extension in local part
# delimiter2 is not set by default
# third delimiter for VERP extension in local part
# delimiter3 is not set by default
}
DEFEDB {
# Home directory for DEFEDB
# base_directory is not set by default
# Log directory for DEFEDB
# log_directory is not set by default
# DB page size
# page_size is not set by default
# DB cache size
cache_size=4 *1024*1024;
# If non-zero, a checkpoint will be done if more than the amount of KBytes
# of log data have been written since the last checkpoint
# KBytes_written_for_checkpointing is not set by default
# Minimum delay between two checkpoints
delay_between_2_checkpoints=600;
# flags for DEFEDB
# flags are not set by default
# initial size for in-memory cache
hash_table_entries=${EDBC_HT_INIT};
}
# name of control socket
# control_socket is not set by default
# name of configuration map (including extension)
conf="qmgr_conf.db";
# full address for double bounces (defaults to <doublebounce@HOSTNAME>)
# double_bounce_address is not set by default
# minimum number of threads [experts only]
threads_min=2;
# maximum number of threads [experts only]
threads_max=6;
# maximum number of file descriptors [experts only]
fds_max=10;
# minimum number of processes
processes_min=1;
# maximum number of processes
processes_max=1;
# run process as user
user="";
# run process as group
group="";
# list of services to restart in case of failure
# restart_dependencies is not set by default
# pass id to process in argument list with this option
# pass_id is not set by default
# use id to create name of logfile: Logdir name id . log
use_id_in_logfile_name=0;
# path to program
path="";
# arguments for program
arguments="";
# working directory (chdir(2))
# working_directory is not set by default
# maximum number of failures
# failure_max is not set by default
# how to handle program
start_action="wait";
}
EOF

# see "dirty hack" above
if ${PRG} ${CNF} | sed -e 's/^  *//' -e '/^$/d' -e '/^options=/d' >${OUT}
then
  if diff ${OK} ${OUT}
  then
    :
  else
    ERRS=`expr ${ERRS} + 1 `
  fi
else
  ERRS=`expr ${ERRS} + 1 `
fi

# ----------------
if test "${ERRS}" = "0"
then
  exit 0
else
  echo "${ERRS} error(s)"
  exit 1
fi
