#include "sm/generic.h"
SM_RCSID("@(#)$Id: sm_inet_ipv6f.c,v 1.4 2013/09/22 20:23:10 ca Exp $")

#include "sm/assert.h"
#include "sm/error.h"
#include "sm/ctype.h"
#include "sm/net.h"
#include "sm/io.h"
#include "sm/str.h"
#include "sm/nameser.h"	/* some INADDR constants */

#if MTA_NETINET6
/*
**  SM_INET_IPV6FSTR -- convert IPv6 address to ASCII string (uncompressed)
**
**	Parameters:
**		ipv6 -- IPv6 address
**		dst -- append ASCII representation of address (output)
**
**	Returns:
**		error: ENOSPC
*/

sm_ret_T
sm_inet_ipv6fstr(ipv6_P ipv6, sm_str_P dst)
{
	uint16_t *u16;
	int r;

	u16 = (uint16_t*)ipv6;
	r = sm_strprintf(dst,
		"%x:%x:%x:%x:%x:%x:%x:%x"
			, htons(u16[0])
			, htons(u16[1])
			, htons(u16[2])
			, htons(u16[3])
			, htons(u16[4])
			, htons(u16[5])
			, htons(u16[6])
			, htons(u16[7])
		);
	if (r > 0) return SM_SUCCESS;
	return sm_err_perm(ENOSPC);
}

/*
**  SM_INET_IN6FADDRSTR -- convert an IPv6 address struct to an ASCII string
**
**	Parameters:
**		in6addr -- IPv6 address struct
**		dst -- append ASCII representation of address (output)
**
**	Returns:
**		error: ENOSPC
*/

sm_ret_T
sm_inet_in6faddrstr(struct in6_addr in6addr, sm_str_P dst)
{
	uchar *src;
	ipv6_T ipv6;

	src = (uchar *)&in6addr;
	sm_memcpy(&ipv6, src, sizeof(ipv6));
	return sm_inet_ipv6fstr(&ipv6, dst);
}
#endif /* MTA_NETINET6 */
