/*
 *  test reopening sequence map
 *  See comments below about reopen.
 */

#include "sm/generic.h"
SM_RCSID("@(#)$Id: t-map-reopen-1.c,v 1.8 2010/08/31 03:26:32 ca Exp $")

#include "sm/error.h"
#include "sm/heap.h"
#include "sm/memops.h"
#include "sm/test.h"
#include "sm/maps.h"
#include "sm/mapc.h"
#include "sm/map.h"
#include "sm/mapclasses.h"
#include "sm/bdb.h"

#include "sm/io.h"

int Verbose = 0;

#include "t-seq-common.c"

#define MAPC_TYPE2R	"hash"
#define MAPC_NAME2R	"bdbr"
#define MAPC_FILE2R	"./bdbrr0.db"
#define MAPC_FILEB	"./bdbrb0.db"
#define DATAR1	"reopen1"
#define DATAR2	"reopen2"

static void
testh(bool fail)
{
	sm_ret_T ret;
	sm_maps_P maps;
	sm_map_P map1, map2, map2r, map3;
	sm_cstr_P mtype, mname;
	sm_cstr_P mname1, mname2;
	sm_str_P lhs, rhs;
	sm_str_T key, data;
	int i;

	maps = NULL;
	mtype = mname = NULL;
	mname1 = mname2 = NULL;
	lhs = rhs = NULL;

	ret = sm_maps_init(&maps);
	SM_TEST(maps != NULL);
	if (NULL == maps)
		return;
	SM_TEST(sm_is_success(ret));
	ret = sm_bht_class_create(maps);
	SM_TEST_E(sm_is_success(ret));
	ret = sm_seq_class_create(maps);
	SM_TEST_E(sm_is_success(ret));

	ret = crt_map(maps, &map1, MAPC_NAME1, MAPC_TYPE1);
	SM_TEST_E(sm_is_success(ret));
	ret = crt_bdb_map(maps, &map2, MAPC_NAME2, MAPC_TYPE2, MAPC_FILE2);
	SM_TEST_E(sm_is_success(ret));
	ret = crt_bdb_map(maps, &map2r, MAPC_NAME2R, MAPC_TYPE2R, MAPC_FILE2R);
	SM_TEST_E(sm_is_success(ret));

	SM_POP_MAP1;
	SM_POP_MAP2;
	SM_POP_BDB2;

	/* need to close maps so they are written to disk. can we have a "sync" function? */
	ret = sm_map_close(map2, 0);
	SM_TEST_E(sm_is_success(ret));
	ret = sm_map_close(map2r, 0);
	SM_TEST_E(sm_is_success(ret));

	mtype = sm_cstr_scpyn0((const uchar *)MAPC_TYPE2, strlen(MAPC_TYPE2));
	SM_TEST_E(mtype != NULL);
	lhs = sm_str_new(NULL, 256, 1024);
	SM_TEST_E(lhs != NULL);
	rhs = sm_str_new(NULL, 256, 1024);
	SM_TEST_E(rhs != NULL);
	ret = sm_bdb_class_create(maps);
	SM_TEST(sm_is_success(ret));
	mname1 = sm_cstr_scpyn0((const uchar *)MAPC_NAME1, strlen(MAPC_NAME1));
	ret = sm_error_temp(SM_EM_MAP, ENOMEM);
	SM_TEST_E(mname1 != NULL);
	mname2 = sm_cstr_scpyn0((const uchar *)MAPC_NAME2, strlen(MAPC_NAME2));
	ret = sm_error_temp(SM_EM_MAP, ENOMEM);
	SM_TEST_E(mname2 != NULL);
	mname = sm_cstr_scpyn0((const uchar *)MAPC_NAME2R, strlen(MAPC_NAME2R));
	SM_TEST_E(mname != NULL);

	map2 = NULL;
	ret = sm_map_open(maps, mname2, mtype, SMAP_MODE_RDONLY, MAPC_FILE2,
			SMAP_MODE_RDONLY, &map2, SMPO_END);
	SM_TEST_E(sm_is_success(ret));
	map2r = NULL;
	ret = sm_map_open(maps, mname, mtype, SMAP_MODE_RDONLY, MAPC_FILE2R,
			SMAP_MODE_RDONLY, &map2r, SMPO_END);
	SM_TEST_E(sm_is_success(ret));

	/* create sequence map */
	mtype = mname = NULL;
	mtype = sm_cstr_scpyn0((const uchar *)MAPC_TYPE3, strlen(MAPC_TYPE3));
	ret = sm_error_temp(SM_EM_MAP, ENOMEM);
	SM_TEST_E(mtype != NULL);
	mname = sm_cstr_scpyn0((const uchar *)MAPC_NAME3, strlen(MAPC_NAME3));
	SM_TEST_E(mname != NULL);
	map3 = NULL;
	ret = sm_map_create(maps, mtype, 0, &map3);
	SM_TEST_E(sm_is_success(ret));
	ret = sm_map_setopt(map3,
			SMPO_MAP, map1,
			SMPO_MAP, map2,
			SMPO_END);
	SM_TEST_E(sm_is_success(ret));
	ret = sm_map_open(maps, mname, mtype, 0, NULL, SMAP_MODE_RDWR, &map3,
			SMPO_END);
	SM_TEST_E(sm_is_success(ret));
	SM_CSTR_FREE(mtype);
	SM_CSTR_FREE(mname);

	/* perform some operations ... */
	SM_LOOKUP_TEST(map3, KEY1_1, KEY1_2, KEY1_3);

	SM_REWRITE_TEST(map3, KEY1_1, DATA1_1);
	SM_REWRITE_TEST(map3, KEY1_2, DATA1_2);
	SM_REWRITE_TEST(map3, KEY2_1, DATA2_1);
	SM_REWRITE_TEST(map3, KEY2_2, DATA2_2);

	SM_REWRITE_TEST(map2r, KEY2_1, DATAR1);
	SM_REWRITE_TEST(map2r, KEY2_2, DATAR2);

	i = rename(MAPC_FILE2, MAPC_FILEB);
	SM_TEST(i == 0);

	if (fail) {
		/* shouldn't exist anymore */
		(void) unlink(MAPC_FILE2);
	}
	else {
		i = rename(MAPC_FILE2R, MAPC_FILE2);
		SM_TEST(i == 0);
	}

	/* should a sequence map reopen all of its components? */
	ret = sm_map_reopen(map3, 0, SMPO_END);
	if (fail) {
		SM_TEST_E(sm_is_err(ret));

		sm_str_clr(lhs);
		sm_str_clr(rhs);
		ret = sm_str_scat(lhs, KEY1_1);
		SM_TEST(sm_is_success(ret));
		ret = sm_map_rewrite(map3, SMMAP_FL_NONE, lhs, rhs);
		SM_TEST(sm_is_err(ret));

		i = rename(MAPC_FILE2R, MAPC_FILE2);
		SM_TEST(i == 0);

		ret = sm_map_reopen(map3, 0, SMPO_END);
	}

	SM_TEST_E(sm_is_success(ret));

#if 0
	ret = sm_map_reopen(map2, 0, SMPO_END);
	SM_TEST_E(sm_is_success(ret));
#endif

	/* perform some operations ... */
	SM_REWRITE_TEST(map3, KEY1_1, DATA1_1);
	SM_REWRITE_TEST(map3, KEY1_2, DATA1_2);
	SM_REWRITE_TEST(map3, KEY2_1, DATAR1);
	SM_REWRITE_TEST(map3, KEY2_2, DATAR2);

	sm_str_clr(lhs);
	sm_str_clr(rhs);
	ret = sm_str_scat(lhs, KEY1_3);
	SM_TEST(sm_is_success(ret));
	ret = sm_map_lookup(map3, SMMAP_FL_NONE, lhs, rhs);
	SM_TEST(sm_is_err(ret));

	ret = sm_map_close(map3, 0);
	SM_TEST(sm_is_success(ret));

	ret = sm_maps_term(maps);
	SM_TEST(sm_is_success(ret));
	SM_STR_FREE(lhs);
	SM_STR_FREE(rhs);
	return;

  error:
	sm_maps_term(maps);
}


int
main(int argc, char *argv[])
{
	int c;
	bool fail;

	fail = false;
	while ((c = getopt(argc, argv, "f")) != -1) {
		switch (c) {
		  case 'f':
			fail = true;
			break;
		  default:
			return 1;
		}
	}
	sm_test_begin(argc, argv, "test map reopen 1");
	testh(fail);
	return sm_test_end();
}
